@@ -44,6 +44,8 @@ from cinder.volume import volume_utils
 
 if sys.platform == 'win32':
     from os_win import utilsfactory as os_win_utilsfactory
+else:
+    os_win_utilsfactory = None
 
 LOG = logging.getLogger(__name__)
 
@@ -66,6 +68,27 @@ CONF = cfg.CONF
 CONF.register_opts(backup_opts)
 
 
+def _write_nonzero(volume_file, volume_offset, content):
+    """Write non-zero parts of `content` into `volume_file`."""
+    chunk_length = 1024 * 1024
+    content = memoryview(content)
+    for chunk_offset in range(0, len(content), chunk_length):
+        chunk_end = chunk_offset + chunk_length
+        chunk = content[chunk_offset:chunk_end]
+        # The len(chunk) may be smaller than chunk_length. It's okay.
+        if not volume_utils.is_all_zero(chunk):
+            volume_file.seek(volume_offset + chunk_offset)
+            volume_file.write(chunk)
+
+
+def _write_volume(volume_is_new, volume_file, volume_offset, content):
+    if volume_is_new:
+        _write_nonzero(volume_file, volume_offset, content)
+    else:
+        volume_file.seek(volume_offset)
+        volume_file.write(content)
+
+
 # Object writer and reader returned by inheriting classes must not have any
 # logging calls, as well as the compression libraries, as eventlet has a bug
 # (https://github.com/eventlet/eventlet/issues/432) that would result in
@@ -535,6 +558,7 @@ class ChunkedBackupDriver(driver.BackupDriver, metaclass=abc.ABCMeta):
                         'backup. Do a full backup.')
                 raise exception.InvalidBackup(reason=err)
 
+        win32_disk_size = None
         if sys.platform == 'win32':
             # When dealing with Windows physical disks, we need the exact
             # size of the disk. Attempting to read passed this boundary will
@@ -582,7 +606,7 @@ class ChunkedBackupDriver(driver.BackupDriver, metaclass=abc.ABCMeta):
                 break
             data_offset = volume_file.tell()
 
-            if sys.platform == 'win32':
+            if win32_disk_size is not None:
                 read_bytes = min(self.chunk_size_bytes,
                                  win32_disk_size - data_offset)
             else:
@@ -666,7 +690,7 @@ class ChunkedBackupDriver(driver.BackupDriver, metaclass=abc.ABCMeta):
         self._finalize_backup(backup, container, object_meta, object_sha256)
 
     def _restore_v1(self, backup, volume_id, metadata, volume_file,
-                    requested_backup):
+                    volume_is_new, requested_backup):
         """Restore a v1 volume backup.
 
         Raises BackupRestoreCancel on any requested_backup status change, we
@@ -717,16 +741,17 @@ class ChunkedBackupDriver(driver.BackupDriver, metaclass=abc.ABCMeta):
                 body = reader.read()
             compression_algorithm = metadata_object[object_name]['compression']
             decompressor = self._get_compressor(compression_algorithm)
-            volume_file.seek(obj['offset'])
             if decompressor is not None:
                 LOG.debug('decompressing data using %s algorithm',
                           compression_algorithm)
                 decompressed = decompressor.decompress(body)
                 body = None  # Allow Python to free it
-                volume_file.write(decompressed)
+                _write_volume(volume_is_new,
+                              volume_file, obj['offset'], decompressed)
                 decompressed = None  # Allow Python to free it
             else:
-                volume_file.write(body)
+                _write_volume(volume_is_new,
+                              volume_file, obj['offset'], body)
                 body = None  # Allow Python to free it
 
             # force flush every write to avoid long blocking write on close
@@ -748,7 +773,7 @@ class ChunkedBackupDriver(driver.BackupDriver, metaclass=abc.ABCMeta):
         LOG.debug('v1 volume backup restore of %s finished.',
                   backup_id)
 
-    def restore(self, backup, volume_id, volume_file):
+    def restore(self, backup, volume_id, volume_file, volume_is_new):
         """Restore the given volume backup from backup repository.
 
         Raises BackupRestoreCancel on any backup status change.
@@ -757,13 +782,15 @@ class ChunkedBackupDriver(driver.BackupDriver, metaclass=abc.ABCMeta):
         container = backup['container']
         object_prefix = backup['service_metadata']
         LOG.debug('starting restore of backup %(object_prefix)s '
-                  'container: %(container)s, to volume %(volume_id)s, '
+                  'container: %(container)s, '
+                  'to %(new)s volume %(volume_id)s, '
                   'backup: %(backup_id)s.',
                   {
                       'object_prefix': object_prefix,
                       'container': container,
                       'volume_id': volume_id,
                       'backup_id': backup_id,
+                      'new': 'new' if volume_is_new else 'existing',
                   })
         metadata = self._read_metadata(backup)
         metadata_version = metadata['version']
@@ -794,7 +821,8 @@ class ChunkedBackupDriver(driver.BackupDriver, metaclass=abc.ABCMeta):
             backup1 = backup_list[index]
             index = index - 1
             metadata = self._read_metadata(backup1)
-            restore_func(backup1, volume_id, metadata, volume_file, backup)
+            restore_func(backup1, volume_id, metadata, volume_file,
+                         volume_is_new, backup)
 
             volume_meta = metadata.get('volume_meta', None)
             try:
