@@ -21,6 +21,8 @@ import stat
 from os_brick.remotefs import remotefs as remotefs_brick
 from oslo_concurrency import processutils as putils
 from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_log import versionutils
 
 from cinder.backup.drivers import posix
 from cinder import exception
@@ -38,6 +40,8 @@ glusterfsbackup_service_opts = [
                     'Eg: 1.2.3.4:backup_vol'),
 ]
 
+LOG = logging.getLogger(__name__)
+
 CONF = cfg.CONF
 CONF.register_opts(glusterfsbackup_service_opts)
 
@@ -46,14 +50,13 @@ CONF.register_opts(glusterfsbackup_service_opts)
 class GlusterfsBackupDriver(posix.PosixBackupDriver):
     """Provides backup, restore and delete using GlusterFS repository."""
 
-    def __init__(self, context, db=None):
+    def __init__(self, context):
         self.backup_mount_point_base = CONF.glusterfs_backup_mount_point
         self.backup_share = CONF.glusterfs_backup_share
         self._execute = putils.execute
         self._root_helper = utils.get_root_helper()
         backup_path = self._init_backup_repo_path()
-        super(GlusterfsBackupDriver, self).__init__(context,
-                                                    backup_path=backup_path)
+        super().__init__(context, backup_path=backup_path)
 
     @staticmethod
     def get_driver_options():
@@ -61,6 +64,12 @@ class GlusterfsBackupDriver(posix.PosixBackupDriver):
 
     def check_for_setup_error(self):
         """Raises error if any required configuration flag is missing."""
+
+        versionutils.report_deprecated_feature(
+            LOG,
+            "The Cinder GlusterFS Backup Driver is deprecated and will be "
+            "removed in the 2025.1 release.")
+
         required_flags = ['glusterfs_backup_share']
         for flag in required_flags:
             val = getattr(CONF, flag, None)
