@@ -22,7 +22,7 @@ SHOULD include dedicated exception logging.
 
 """
 
-from typing import Optional, Union  # noqa: H301
+from typing import Optional, Union
 
 from oslo_log import log as logging
 from oslo_versionedobjects import exception as obj_exc
@@ -1094,6 +1094,10 @@ class DriverInitiatorDataExists(Duplicate):
     )
 
 
+class RequirementMissing(CinderException):
+    message = _('Requirement %(req)s is not installed.')
+
+
 class ConflictNovaUsingAttachment(CinderException):
     message = _("Detach volume from instance %(instance_id)s using the "
                 "Compute API")
