@@ -35,7 +35,15 @@ class VolumeDriverCore(base.CinderInterface):
         Can be used to set up clients, check licenses, set up protocol
         specific helpers, etc.
 
-        :param context: The admin context.
+        If you choose to raise an exception here, the setup is considered
+        failed already and the check_for_setup_error() will not be called.
+
+        :param context: The admin context of type context.RequestContext.
+        :raises InvalidConfigurationValue: raise this if you detect a problem
+                                           during a configuration check
+        :raises VolumeDriverException: raise this or one of its more specific
+                                       subclasses if you detect setup problems
+                                       other than invalid configuration
         """
 
     def check_for_setup_error(self):
@@ -53,6 +61,8 @@ class VolumeDriverCore(base.CinderInterface):
         external dependencies are present and working.
 
         :raises VolumeBackendAPIException: in case of setup error.
+        :raises InvalidConfigurationValue: raise this if you detect a problem
+                                           during a configuration check
         """
 
     def get_volume_stats(self, refresh=False):
@@ -150,6 +160,9 @@ class VolumeDriverCore(base.CinderInterface):
         * online_extend_support (Boolean)
             Whether the backend supports in-use volume extend or not. Defaults
             to True.
+        * clone_across_pools (Boolean)
+            Whether the backend supports cloning a volume across different
+            pools. Defaults to False.
 
         The returned dict may also contain a list, "pools", which has a similar
         dict for each pool being used with the backend.
@@ -291,13 +304,15 @@ class VolumeDriverCore(base.CinderInterface):
                   whether the clone occurred.
         """
 
-    def copy_image_to_volume(self, context, volume, image_service, image_id):
+    def copy_image_to_volume(self, context, volume, image_service, image_id,
+                             disable_sparse=False):
         """Fetch the image from image_service and write it to the volume.
 
         :param context: Security/policy info for the request.
         :param volume: The volume to create.
         :param image_service: The image service to use.
         :param image_id: The image identifier.
+        :param disable_sparse: Enable or disable sparse copy. Default=False.
         :returns: Model updates.
         """
 
@@ -311,6 +326,32 @@ class VolumeDriverCore(base.CinderInterface):
         :returns: Model updates.
         """
 
+    def before_volume_copy(self, context, src_vol, dest_vol, remote=None):
+        """Driver-specific actions executed before copying a volume.
+
+        This method will be called before _copy_volume_data during volume
+        migration.
+
+        :param context: Context
+        :param src_volume: Source volume in the copy operation.
+        :param dest_volume: Destination volume in the copy operation.
+        :param remote: Whether the copy operation is local.
+        :returns: There is no return value for this method.
+        """
+
+    def after_volume_copy(self, context, src_vol, dest_vol, remote=None):
+        """Driver-specific actions executed after copying a volume.
+
+        This method will be called after _copy_volume_data during volume
+        migration.
+
+        :param context: Context
+        :param src_volume: Source volume in the copy operation.
+        :param dest_volume: Destination volume in the copy operation.
+        :param remote: Whether the copy operation is local.
+        :returns: There is no return value for this method.
+        """
+
     def extend_volume(self, volume, new_size):
         """Extend the size of a volume.
 
@@ -321,6 +362,52 @@ class VolumeDriverCore(base.CinderInterface):
         volume, the driver should report online_extend_support=False.
         """
 
+    def migrate_volume(self, context, volume, host):
+        """Migrate the volume to the specified host.
+
+        :param context: Context
+        :param volume: A dictionary describing the volume to migrate
+        :param host: A dictionary describing the host to migrate to, where
+                     host['host'] is its name, and host['capabilities'] is a
+                     dictionary of its reported capabilities.
+        :returns: Tuple of (model_update, boolean) where the boolean specifies
+                  whether the migration occurred.
+        """
+
+    def update_migrated_volume(self, context, volume, new_volume,
+                               original_volume_status):
+        """Return model update for migrated volume.
+
+        Each driver implementing this method needs to be responsible for the
+        values of _name_id and provider_location. If None is returned or either
+        key is not set, it means the volume table does not need to change the
+        value(s) for the key(s).
+        The return format is {"_name_id": value, "provider_location": value}.
+
+        :param context: Context
+        :param volume: The original volume that was migrated to this backend
+        :param new_volume: The migration volume object that was created on
+                           this backend as part of the migration process
+        :param original_volume_status: The status of the original volume
+        :returns: model_update to update DB with any needed changes
+        """
+
+    def retype(self, context, volume, new_type, diff, host):
+        """Change the type of a volume.
+
+        This operation occurs on the same backend and the return value
+        indicates whether it was successful.  If migration is required
+        to satisfy a retype, that will be handled by the volume manager.
+
+        :param context: Context
+        :param volume: The volume to retype
+        :param new_type: The target type for the volume
+        :param diff: The differences between the two types
+        :param host: The host that contains this volume
+        :returns: Tuple of (boolean, model_update) where the boolean specifies
+                  whether the retype occurred.
+        """
+
     def create_snapshot(self, snapshot):
         """Creates a snapshot.
 
@@ -348,3 +435,63 @@ class VolumeDriverCore(base.CinderInterface):
         :param snapshot: The snapshot from which to create the volume.
         :returns: A dict of database updates for the new volume.
         """
+
+    def set_initialized(self):
+        """Mark driver as initialized.
+
+        Do not implement this in a driver. Rely on the default implementation.
+        """
+
+    def initialized(self):
+        """Getter for driver's initialized status.
+
+        Do not implement this in a driver. Rely on the default implementation.
+        """
+
+    def supported(self):
+        """Getter for driver's supported status.
+
+        Do not implement this in a driver. Rely on the default implementation.
+        """
+
+    def set_throttle(self):
+        """Hook for initialization of cinder.volume.throttle.
+
+        This has not been necessary to re-implement or override in any
+        drivers thus far. The generic implementation does nothing unless
+        explicitly enabled.
+        """
+
+    def init_capabilities(self):
+        """Fetch and merge capabilities of the driver.
+
+        Do not override this, implement _init_vendor_properties instead.
+        """
+
+    def _init_vendor_properties(self):
+        """Create a dictionary of vendor unique properties.
+
+        Compose a dictionary by calling ``self._set_property``.
+
+        Select a prefix from the vendor, product, or device name.
+        Prefix must match the part of property name before colon (:).
+
+        :returns tuple (properties: dict, prefix: str)
+        """
+
+    def update_provider_info(self, volumes, snapshots):
+        """Get provider info updates from driver.
+
+        This retrieves a list of volumes and a list of snapshots that
+        changed their providers thanks to the initialization of the host,
+        so that Cinder can update this information in the volume database.
+
+        This is only implemented by drivers where such migration is possible.
+
+        :param volumes: List of Cinder volumes to check for updates
+        :param snapshots: List of Cinder snapshots to check for updates
+        :returns: tuple (volume_updates, snapshot_updates)
+
+        where volume updates {'id': uuid, provider_id: <provider-id>}
+        and snapshot updates {'id': uuid, provider_id: <provider-id>}
+        """
