@@ -13,13 +13,13 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
+from http import HTTPStatus
 from unittest import mock
+from urllib.parse import urlencode
 
 from oslo_config import cfg
 import oslo_messaging as messaging
 from oslo_serialization import jsonutils
-from six.moves import http_client
-from six.moves.urllib.parse import urlencode
 import webob
 
 from cinder.common import constants
@@ -36,9 +36,9 @@ CONF = cfg.CONF
 
 def app():
     # no auth, just let environ['cinder.context'] pass through
-    api = fakes.router.APIRouter()
+    api = fakes.router_v3.APIRouter()
     mapper = fakes.urlmap.URLMap()
-    mapper['/v2'] = api
+    mapper['/v3'] = api
     return mapper
 
 
@@ -100,7 +100,7 @@ class SnapshotManageTest(test.TestCase):
 
     def _get_resp_post(self, body):
         """Helper to execute an os-snapshot-manage API call."""
-        req = webob.Request.blank('/v2/%s/os-snapshot-manage' %
+        req = webob.Request.blank('/v3/%s/os-snapshot-manage' %
                                   fake.PROJECT_ID)
         req.method = 'POST'
         req.headers['Content-Type'] = 'application/json'
@@ -111,7 +111,8 @@ class SnapshotManageTest(test.TestCase):
 
     @mock.patch(
         'cinder.scheduler.rpcapi.SchedulerAPI.manage_existing_snapshot')
-    @mock.patch('cinder.volume.api.API.create_snapshot_in_db')
+    @mock.patch('cinder.volume.api.API.create_snapshot_in_db',
+                return_value=mock.MagicMock(id=fake.SNAPSHOT_ID))
     @mock.patch('cinder.db.sqlalchemy.api.service_get')
     def test_manage_snapshot_ok(self, mock_db,
                                 mock_create_snapshot, mock_rpcapi):
@@ -130,7 +131,7 @@ class SnapshotManageTest(test.TestCase):
                              'ref': {'fake_key': 'fake_ref'}}}
 
         res = self._get_resp_post(body)
-        self.assertEqual(http_client.ACCEPTED, res.status_int, res)
+        self.assertEqual(HTTPStatus.ACCEPTED, res.status_int, res)
 
         # Check the db.service_get was called with correct arguments.
         mock_db.assert_called_once_with(
@@ -148,37 +149,38 @@ class SnapshotManageTest(test.TestCase):
         # correct arguments.
         self.assertEqual(1, mock_rpcapi.call_count)
         args = mock_rpcapi.call_args[0]
-        self.assertEqual({u'fake_key': u'fake_ref'}, args[3])
+        self.assertEqual({'fake_key': 'fake_ref'}, args[3])
 
     @mock.patch(
         'cinder.scheduler.rpcapi.SchedulerAPI.manage_existing_snapshot')
-    @mock.patch('cinder.volume.api.API.create_snapshot_in_db')
+    @mock.patch('cinder.volume.api.API.create_snapshot_in_db',
+                return_value=mock.MagicMock(id=fake.SNAPSHOT_ID))
     @mock.patch('cinder.objects.service.Service.get_by_id')
     def test_manage_snapshot_ok_with_metadata_null(
             self, mock_db, mock_create_snapshot, mock_rpcapi):
         mock_db.return_value = fake_service.fake_service_obj(
             self._admin_ctxt,
             binary=constants.VOLUME_BINARY)
+
         body = {'snapshot': {'volume_id': fake.VOLUME_ID,
                              'ref': {'fake_key': 'fake_ref'},
                              'name': 'test',
                              'description': 'test',
                              'metadata': None}}
-
         res = self._get_resp_post(body)
-        self.assertEqual(http_client.ACCEPTED, res.status_int)
+        self.assertEqual(HTTPStatus.ACCEPTED, res.status_int)
         args = mock_create_snapshot.call_args[0]
         # 5th argument of args is metadata.
         self.assertIsNone(args[5])
 
     @mock.patch(
         'cinder.scheduler.rpcapi.SchedulerAPI.manage_existing_snapshot')
-    @mock.patch('cinder.volume.api.API.create_snapshot_in_db')
+    @mock.patch('cinder.volume.api.API.create_snapshot_in_db',
+                return_value=mock.MagicMock(id=fake.SNAPSHOT_ID))
     @mock.patch('cinder.db.sqlalchemy.api.service_get')
     def test_manage_snapshot_ok_ref_as_string(self, mock_db,
                                               mock_create_snapshot,
                                               mock_rpcapi):
-
         mock_db.return_value = fake_service.fake_service_obj(
             self._admin_ctxt,
             binary=constants.VOLUME_BINARY)
@@ -187,7 +189,7 @@ class SnapshotManageTest(test.TestCase):
                              'ref': "string"}}
 
         res = self._get_resp_post(body)
-        self.assertEqual(http_client.ACCEPTED, res.status_int, res)
+        self.assertEqual(HTTPStatus.ACCEPTED, res.status_int, res)
 
         # Check the volume_rpcapi.manage_existing_snapshot was called with
         # correct arguments.
@@ -199,17 +201,20 @@ class SnapshotManageTest(test.TestCase):
                 return_value=True,
                 new_callable=mock.PropertyMock)
     @mock.patch('cinder.volume.rpcapi.VolumeAPI.manage_existing_snapshot')
-    @mock.patch('cinder.volume.api.API.create_snapshot_in_db')
+    @mock.patch('cinder.volume.api.API.create_snapshot_in_db',
+                return_value=mock.MagicMock(id=fake.SNAPSHOT_ID))
     @mock.patch('cinder.db.sqlalchemy.api.service_get')
     def test_manage_snapshot_disabled(self, mock_db, mock_create_snapshot,
                                       mock_rpcapi, mock_is_up):
         """Test manage snapshot failure due to disabled service."""
         mock_db.return_value = fake_service.fake_service_obj(self._admin_ctxt,
                                                              disabled=True)
+
         body = {'snapshot': {'volume_id': fake.VOLUME_ID, 'ref': {
             'fake_key': 'fake_ref'}}}
+
         res = self._get_resp_post(body)
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int, res)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int, res)
         self.assertEqual(exception.ServiceUnavailable.message,
                          res.json['badRequest']['message'])
         mock_create_snapshot.assert_not_called()
@@ -219,7 +224,8 @@ class SnapshotManageTest(test.TestCase):
     @mock.patch('cinder.objects.service.Service.is_up', return_value=False,
                 new_callable=mock.PropertyMock)
     @mock.patch('cinder.volume.rpcapi.VolumeAPI.manage_existing_snapshot')
-    @mock.patch('cinder.volume.api.API.create_snapshot_in_db')
+    @mock.patch('cinder.volume.api.API.create_snapshot_in_db',
+                return_value=mock.MagicMock(id=fake.SNAPSHOT_ID))
     @mock.patch('cinder.db.sqlalchemy.api.service_get')
     def test_manage_snapshot_is_down(self, mock_db, mock_create_snapshot,
                                      mock_rpcapi, mock_is_up):
@@ -228,7 +234,7 @@ class SnapshotManageTest(test.TestCase):
         body = {'snapshot': {'volume_id': fake.VOLUME_ID,
                              'ref': {'fake_key': 'fake_ref'}}}
         res = self._get_resp_post(body)
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int, res)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int, res)
         self.assertEqual(exception.ServiceUnavailable.message,
                          res.json['badRequest']['message'])
         mock_create_snapshot.assert_not_called()
@@ -239,25 +245,25 @@ class SnapshotManageTest(test.TestCase):
         """Test correct failure when volume_id is not specified."""
         body = {'snapshot': {'ref': 'fake_ref'}}
         res = self._get_resp_post(body)
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
 
     def test_manage_snapshot_missing_ref(self):
         """Test correct failure when the ref is not specified."""
         body = {'snapshot': {'volume_id': fake.VOLUME_ID}}
         res = self._get_resp_post(body)
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
 
     def test_manage_snapshot_error_body(self):
         """Test correct failure when body is invaild."""
         body = {'error_snapshot': {'volume_id': fake.VOLUME_ID}}
         res = self._get_resp_post(body)
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
 
     def test_manage_snapshot_error_volume_id(self):
         """Test correct failure when volume id is invalid format."""
         body = {'snapshot': {'volume_id': 'error_volume_id', 'ref': {}}}
         res = self._get_resp_post(body)
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
         self.assertIn("'error_volume_id' is not a 'uuid'",
                       jsonutils.loads(res.body)['badRequest']['message'])
 
@@ -271,7 +277,7 @@ class SnapshotManageTest(test.TestCase):
         detail = ""
         if detailed:
             detail = "/detail"
-        url = "/v2/%s/os-snapshot-manage%s%s" % (fake.PROJECT_ID, detail,
+        url = "/v3/%s/os-snapshot-manage%s%s" % (fake.PROJECT_ID, detail,
                                                  query_string)
         req = webob.Request.blank(url)
         req.method = 'GET'
@@ -285,10 +291,10 @@ class SnapshotManageTest(test.TestCase):
                 wraps=api_get_manageable_snapshots)
     def test_get_manageable_snapshots_non_admin(self, mock_api_manageable):
         res = self._get_resp_get('fakehost', False, False, admin=False)
-        self.assertEqual(http_client.FORBIDDEN, res.status_int)
+        self.assertEqual(HTTPStatus.FORBIDDEN, res.status_int)
         self.assertEqual(False, mock_api_manageable.called)
         res = self._get_resp_get('fakehost', True, False, admin=False)
-        self.assertEqual(http_client.FORBIDDEN, res.status_int)
+        self.assertEqual(HTTPStatus.FORBIDDEN, res.status_int)
         self.assertEqual(False, mock_api_manageable.called)
 
     @mock.patch('cinder.volume.api.API.get_manageable_snapshots',
@@ -304,7 +310,7 @@ class SnapshotManageTest(test.TestCase):
                 {'reference': {'source-name': 'mysnap'}, 'size': 5,
                  'safe_to_manage': True,
                  'source_reference': {'source-name': 'myvol'}}]}
-        self.assertEqual(http_client.OK, res.status_int)
+        self.assertEqual(HTTPStatus.OK, res.status_int)
         self.assertEqual(jsonutils.loads(res.body), exp)
         mock_api_manageable.assert_called_once_with(
             self._admin_ctxt, 'fakehost', None, limit=CONF.osapi_max_limit,
@@ -317,7 +323,7 @@ class SnapshotManageTest(test.TestCase):
     def test_get_manageable_snapshots_non_existent_marker(
             self, mock_api_manageable):
         res = self._get_resp_get('fakehost', detailed=False, paging=True)
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
         self.assertTrue(mock_api_manageable.called)
 
     @mock.patch('cinder.volume.api.API.get_manageable_snapshots',
@@ -336,7 +342,7 @@ class SnapshotManageTest(test.TestCase):
                  'cinder_id': None, 'safe_to_manage': True,
                  'reason_not_safe': None, 'extra_info': 'qos_setting:low',
                  'source_reference': {'source-name': 'myvol'}}]}
-        self.assertEqual(http_client.OK, res.status_int)
+        self.assertEqual(HTTPStatus.OK, res.status_int)
         self.assertEqual(jsonutils.loads(res.body), exp)
         mock_api_manageable.assert_called_once_with(
             self._admin_ctxt, 'fakehost', None, limit=10, marker='1234',
@@ -348,7 +354,7 @@ class SnapshotManageTest(test.TestCase):
     def test_get_manageable_snapshots_non_existent_marker_detailed(
             self, mock_api_manageable):
         res = self._get_resp_get('fakehost', detailed=True, paging=True)
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
         self.assertTrue(mock_api_manageable.called)
 
     @mock.patch('cinder.objects.service.Service.is_up', return_value=True)
@@ -357,7 +363,7 @@ class SnapshotManageTest(test.TestCase):
         mock_db.return_value = fake_service.fake_service_obj(self._admin_ctxt,
                                                              disabled=True)
         res = self._get_resp_get('host_ok', False, True)
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int, res)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int, res)
         self.assertEqual(exception.ServiceUnavailable.message,
                          res.json['badRequest']['message'])
         mock_is_up.assert_not_called()
@@ -368,14 +374,15 @@ class SnapshotManageTest(test.TestCase):
     def test_get_manageable_snapshots_is_down(self, mock_db, mock_is_up):
         mock_db.return_value = fake_service.fake_service_obj(self._admin_ctxt)
         res = self._get_resp_get('host_ok', False, True)
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int, res)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int, res)
         self.assertEqual(exception.ServiceUnavailable.message,
                          res.json['badRequest']['message'])
         self.assertTrue(mock_is_up.called)
 
     @mock.patch(
         'cinder.scheduler.rpcapi.SchedulerAPI.manage_existing_snapshot')
-    @mock.patch('cinder.volume.api.API.create_snapshot_in_db')
+    @mock.patch('cinder.volume.api.API.create_snapshot_in_db',
+                return_value=mock.MagicMock(id=fake.SNAPSHOT_ID))
     @mock.patch('cinder.objects.service.Service.get_by_id')
     def test_manage_snapshot_with_null_validate(
             self, mock_db, mock_create_snapshot, mock_rpcapi):
@@ -388,5 +395,5 @@ class SnapshotManageTest(test.TestCase):
                              'description': None}}
 
         res = self._get_resp_post(body)
-        self.assertEqual(http_client.ACCEPTED, res.status_int, res)
+        self.assertEqual(HTTPStatus.ACCEPTED, res.status_int, res)
         self.assertIn('snapshot', jsonutils.loads(res.body))
