@@ -18,16 +18,12 @@
 from unittest import mock
 
 import ddt
-from oslo_policy import policy as oslo_policy
 
 from cinder.api import microversions as mv
 from cinder.api.v3 import attachments as v3_attachments
 from cinder import context
 from cinder import exception
 from cinder import objects
-from cinder.policies import attachments as attachments_policies
-from cinder.policies import base as base_policy
-from cinder import policy
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import test
@@ -144,33 +140,6 @@ class AttachmentsAPITestCase(test.TestCase):
                           self.controller.update, req,
                           self.attachment1.id, body=body)
 
-    @mock.patch('cinder.coordination.synchronized')
-    @mock.patch.object(objects.VolumeAttachment, 'get_by_id')
-    def test_attachment_operations_not_authorized(self, mock_get, mock_synch):
-        mock_get.return_value = self.attachment1
-        req = fakes.HTTPRequest.blank('/v3/%s/attachments/%s' %
-                                      (fake.PROJECT2_ID, self.attachment1.id),
-                                      version=mv.NEW_ATTACH,
-                                      use_admin_context=False)
-        body = {
-            "attachment":
-                {
-                    "connector": {'fake_key': 'fake_value',
-                                  'host': 'somehost'},
-                },
-        }
-        rules = {attachments_policies.UPDATE_POLICY:
-                 base_policy.RULE_ADMIN_OR_OWNER}
-        policy.set_rules(oslo_policy.Rules.from_dict(rules))
-        self.addCleanup(policy.reset)
-
-        self.assertRaises(exception.NotAuthorized,
-                          self.controller.update, req,
-                          self.attachment1.id, body=body)
-        self.assertRaises(exception.NotAuthorized,
-                          self.controller.delete, req,
-                          self.attachment1.id)
-
     @ddt.data(mv.get_prior_version(mv.RESOURCE_FILTER),
               mv.RESOURCE_FILTER, mv.LIKE_FILTER)
     @mock.patch('cinder.api.common.reject_invalid_filters')
@@ -190,6 +159,8 @@ class AttachmentsAPITestCase(test.TestCase):
     @ddt.data('reserved', 'attached')
     @mock.patch.object(volume_rpcapi.VolumeAPI, 'attachment_delete')
     def test_delete_attachment(self, status, mock_delete):
+        self.patch('cinder.volume.api.API.attachment_deletion_allowed',
+                   return_value=None)
         volume1 = self._create_volume(display_name='fake_volume_1',
                                       project_id=fake.PROJECT_ID)
         attachment = self._create_attachment(
@@ -216,7 +187,7 @@ class AttachmentsAPITestCase(test.TestCase):
     def _create_attachment(self, ctxt=None, volume_uuid=None,
                            instance_uuid=None, mountpoint=None,
                            attach_time=None, detach_time=None,
-                           attach_status=None, attach_mode=None, host=None):
+                           attach_status=None, attach_mode=None, host=''):
         """Create an attachment object."""
         ctxt = ctxt or self.ctxt
         attachment = objects.VolumeAttachment(ctxt)
