@@ -37,6 +37,7 @@ from cinder import db
 from cinder import exception
 from cinder.i18n import _
 from cinder import objects
+from cinder import service_auth
 from cinder.tests.unit.backup import fake_swift_client
 from cinder.tests.unit.backup import fake_swift_client2
 from cinder.tests.unit import fake_constants as fake
@@ -48,7 +49,7 @@ CONF = cfg.CONF
 ANY = mock.ANY
 
 
-def fake_md5(arg):
+def fake_md5(arg, usedforsecurity=False):
     class result(object):
         def hexdigest(self):
             return 'fake-md5-sum'
@@ -299,6 +300,83 @@ class BackupSwiftTestCase(test.TestCase):
                                                 starting_backoff=ANY,
                                                 cacert=ANY)
 
+    def _test_backup_swift_service_auth_headers_no_impact(self):
+        service = swift_dr.SwiftBackupDriver(self.ctxt)
+        self.assertIsNone(service._headers())
+        current = {'some': 'header'}
+        self.assertEqual(service._headers(current), current)
+
+    def test_backup_swift_service_auth_headers_disabled(self):
+        self._test_backup_swift_service_auth_headers_no_impact()
+
+    def test_backup_swift_service_auth_headers_partial_enabled(self):
+        self.override_config('send_service_user_token', True,
+                             group='service_user')
+        self._test_backup_swift_service_auth_headers_no_impact()
+
+    @mock.patch.object(service_auth, 'get_service_auth_plugin')
+    def test_backup_swift_service_auth_headers_enabled(self, mock_plugin):
+        class FakeServiceAuthPlugin:
+            def get_token(self, session):
+                return "fake"
+        self.override_config('send_service_user_token', True,
+                             group='service_user')
+        self.override_config('backup_swift_service_auth', True)
+        mock_plugin.return_value = FakeServiceAuthPlugin()
+        service = swift_dr.SwiftBackupDriver(self.ctxt)
+        expected = {'X-Service-Token': 'fake'}
+        self.assertEqual(service._headers(), expected)
+        expected = {'X-Service-Token': 'fake', 'some': 'header'}
+        self.assertEqual(service._headers({'some': 'header'}), expected)
+
+    @mock.patch.object(fake_swift_client.FakeSwiftConnection, 'put_container')
+    def test_default_backup_swift_create_storage_policy(self, mock_put):
+        service = swift_dr.SwiftBackupDriver(self.ctxt)
+        service.put_container('missing_container')
+        mock_put.assert_called_once_with('missing_container', headers=None)
+
+    @mock.patch.object(fake_swift_client.FakeSwiftConnection, 'put_container')
+    def test_backup_swift_create_storage_policy(self, mock_put):
+        self.override_config('backup_swift_create_storage_policy',
+                             'mypolicy')
+        service = swift_dr.SwiftBackupDriver(self.ctxt)
+        service.put_container('missing_container')
+        mock_put.assert_called_once_with(
+            'missing_container',
+            headers={'X-Storage-Policy': 'mypolicy'}
+        )
+
+    def test_default_backup_swift_create_storage_policy_put_socket_error(self):
+        service = swift_dr.SwiftBackupDriver(self.ctxt)
+        self.assertRaises(exception.SwiftConnectionFailed,
+                          service.put_container,
+                          'missing_container_socket_error_on_put')
+
+    def test_default_backup_swift_create_storage_policy_head_error(self):
+        service = swift_dr.SwiftBackupDriver(self.ctxt)
+        self.assertRaises(exception.SwiftConnectionFailed,
+                          service.put_container, 'unauthorized_container')
+
+    def test_backup_swift_create_storage_policy_head_error(self):
+        self.override_config('backup_swift_create_storage_policy',
+                             'mypolicy')
+        service = swift_dr.SwiftBackupDriver(self.ctxt)
+        self.assertRaises(exception.SwiftConnectionFailed,
+                          service.put_container,
+                          'unauthorized_container')
+
+    def test_default_backup_swift_create_storage_policy_head_sockerr(self):
+        service = swift_dr.SwiftBackupDriver(self.ctxt)
+        self.assertRaises(exception.SwiftConnectionFailed,
+                          service.put_container, 'socket_error_on_head')
+
+    def test_backup_swift_create_storage_policy_head_socket_error(self):
+        self.override_config('backup_swift_create_storage_policy',
+                             'mypolicy')
+        service = swift_dr.SwiftBackupDriver(self.ctxt)
+        self.assertRaises(exception.SwiftConnectionFailed,
+                          service.put_container, 'socket_error_on_head')
+
     def test_backup_uncompressed(self):
         volume_id = '2b9f10a3-42b4-4fdf-b316-000000ceb039'
         self._create_backup_db_entry(volume_id=volume_id)
@@ -721,7 +799,7 @@ class BackupSwiftTestCase(test.TestCase):
             backup = objects.Backup.get_by_id(self.ctxt, fake.BACKUP_ID)
             backup.status = objects.fields.BackupStatus.RESTORING
             backup.save()
-            service.restore(backup, volume_id, volume_file)
+            service.restore(backup, volume_id, volume_file, False)
 
     def test_restore_delta(self):
         volume_id = '04d83506-bcf7-4ff5-9c65-00000051bd2e'
@@ -771,8 +849,7 @@ class BackupSwiftTestCase(test.TestCase):
             backup = objects.Backup.get_by_id(self.ctxt, fake.BACKUP2_ID)
             backup.status = objects.fields.BackupStatus.RESTORING
             backup.save()
-            service.restore(backup, volume_id,
-                            restored_file)
+            service.restore(backup, volume_id, restored_file, False)
             self.assertTrue(filecmp.cmp(self.volume_file.name,
                             restored_file.name))
 
@@ -787,7 +864,7 @@ class BackupSwiftTestCase(test.TestCase):
             backup = objects.Backup.get_by_id(self.ctxt, fake.BACKUP_ID)
             self.assertRaises(exception.SwiftConnectionFailed,
                               service.restore,
-                              backup, volume_id, volume_file)
+                              backup, volume_id, volume_file, False)
 
     def test_restore_unsupported_version(self):
         volume_id = '390db8c1-32d3-42ca-82c9-00000010c703'
@@ -800,7 +877,7 @@ class BackupSwiftTestCase(test.TestCase):
             backup = objects.Backup.get_by_id(self.ctxt, fake.BACKUP_ID)
             self.assertRaises(exception.InvalidBackup,
                               service.restore,
-                              backup, volume_id, volume_file)
+                              backup, volume_id, volume_file, False)
 
     def test_delete(self):
         volume_id = '9ab256c8-3175-4ad8-baa1-0000007f9d31'
@@ -811,6 +888,17 @@ class BackupSwiftTestCase(test.TestCase):
         backup = objects.Backup.get_by_id(self.ctxt, fake.BACKUP_ID)
         service.delete_backup(backup)
 
+    def test_delete_not_found(self):
+        volume_id = '9ab256c8-3175-4ad8-baa1-0000007f9d31'
+        container_name = 'not_found_on_delete'
+        object_prefix = 'test_prefix'
+        self._create_backup_db_entry(volume_id=volume_id,
+                                     container=container_name,
+                                     service_metadata=object_prefix)
+        service = swift_dr.SwiftBackupDriver(self.ctxt)
+        backup = objects.Backup.get_by_id(self.ctxt, fake.BACKUP_ID)
+        service.delete_backup(backup)
+
     def test_delete_wraps_socket_error(self):
         volume_id = 'f74cb6fa-2900-40df-87ac-0000000f72ea'
         container_name = 'socket_error_on_delete'
