@@ -13,10 +13,9 @@
 #    under the License.
 
 from unittest import mock
+from zoneinfo import ZoneInfo
 
 from oslo_utils import timeutils
-import pytz
-import six
 
 from cinder import exception
 from cinder import objects
@@ -104,8 +103,9 @@ class TestGroupSnapshot(test_objects.BaseObjectsTestCase):
         self.assertTrue(group_snapshot.deleted)
         self.assertEqual(fields.GroupSnapshotStatus.DELETED,
                          group_snapshot.status)
-        self.assertEqual(utcnow_mock.return_value.replace(tzinfo=pytz.UTC),
-                         group_snapshot.deleted_at)
+        self.assertEqual(
+            utcnow_mock.return_value.replace(tzinfo=ZoneInfo('UTC')),
+            group_snapshot.deleted_at)
 
     @mock.patch('cinder.objects.group.Group.get_by_id')
     @mock.patch(
@@ -150,14 +150,10 @@ class TestGroupSnapshot(test_objects.BaseObjectsTestCase):
         # value for that field
         group_snapshot.refresh()
         self._compare(self, db_group_snapshot2, group_snapshot)
-        if six.PY3:
-            call_bool = mock.call.__bool__()
-        else:
-            call_bool = mock.call.__nonzero__()
         group_snapshot_get.assert_has_calls(
             [mock.call(self.context,
                        fake.GROUP_SNAPSHOT_ID),
-             call_bool,
+             mock.call.__bool__(),
              mock.call(self.context,
                        fake.GROUP_SNAPSHOT_ID)])
 
