@@ -16,8 +16,6 @@
 from copy import deepcopy
 import random
 
-import six
-
 from cinder import context
 from cinder.objects import fields
 from cinder.objects import group
@@ -79,7 +77,8 @@ class PowerMaxData(object):
     rdf_group_no_2 = '71'
     rdf_group_no_3 = '72'
     rdf_group_no_4 = '73'
-    u4v_version = '92'
+    u4p_version = '92'
+    u4p_100_endpoint = '100'
     storagegroup_name_source = 'Grp_source_sg'
     storagegroup_name_target = 'Grp_target_sg'
     group_snapshot_name = 'Grp_snapshot'
@@ -181,13 +180,13 @@ class PowerMaxData(object):
     provider_location = {'array': array,
                          'device_id': device_id}
 
-    provider_location2 = {'array': six.text_type(array),
+    provider_location2 = {'array': str(array),
                           'device_id': device_id2}
 
-    provider_location3 = {'array': six.text_type(remote_array),
+    provider_location3 = {'array': str(remote_array),
                           'device_id': device_id2}
 
-    provider_location4 = {'array': six.text_type(uni_array),
+    provider_location4 = {'array': str(uni_array),
                           'device_id': device_id}
     provider_location_clone = {'array': array,
                                'device_id': device_id,
@@ -203,7 +202,7 @@ class PowerMaxData(object):
 
     replication_update = (
         {'replication_status': 'enabled',
-         'replication_driver_data': six.text_type(
+         'replication_driver_data': str(
              {'array': remote_array, 'device_id': device_id2})})
 
     legacy_provider_location = {
@@ -226,52 +225,52 @@ class PowerMaxData(object):
 
     test_volume = fake_volume.fake_volume_obj(
         context=ctx, name='vol1', size=2, provider_auth=None,
-        provider_location=six.text_type(provider_location),
+        provider_location=str(provider_location),
         volume_type=test_volume_type, host=fake_host,
-        replication_driver_data=six.text_type(provider_location3))
+        replication_driver_data=str(provider_location3))
 
     test_rep_volume = fake_volume.fake_volume_obj(
         context=ctx, name='vol1', size=2, provider_auth=None,
-        provider_location=six.text_type(provider_location),
+        provider_location=str(provider_location),
         volume_type=test_volume_type, host=fake_host,
-        replication_driver_data=six.text_type(provider_location3),
+        replication_driver_data=str(provider_location3),
         replication_status=fields.ReplicationStatus.ENABLED)
 
     test_attached_volume = fake_volume.fake_volume_obj(
         id='4732de9b-98a4-4b6d-ae4b-3cafb3d34220', context=ctx, name='vol1',
         size=0, provider_auth=None, attach_status='attached',
-        provider_location=six.text_type(provider_location), host=fake_host,
+        provider_location=str(provider_location), host=fake_host,
         volume_type=test_volume_type,
-        replication_driver_data=six.text_type(provider_location3))
+        replication_driver_data=str(provider_location3))
 
     test_legacy_vol = fake_volume.fake_volume_obj(
         context=ctx, name='vol1', size=2, provider_auth=None,
-        provider_location=six.text_type(legacy_provider_location),
-        replication_driver_data=six.text_type(legacy_provider_location2),
+        provider_location=str(legacy_provider_location),
+        replication_driver_data=str(legacy_provider_location2),
         host=fake_host, volume_type=test_volume_type)
 
     test_clone_volume = fake_volume.fake_volume_obj(
         context=ctx, name='vol1', size=2, provider_auth=None,
-        provider_location=six.text_type(provider_location2),
+        provider_location=str(provider_location2),
         host=fake_host, source_volid=test_volume.id,
         snapshot_id=snapshot_id, _name_id=test_volume.id)
 
     test_volume_snap_manage = fake_volume.fake_volume_obj(
         context=ctx, name='vol1', size=2, provider_auth=None,
         display_name='vol1',
-        provider_location=six.text_type(provider_location),
+        provider_location=str(provider_location),
         volume_type=test_volume_type, host=fake_host,
-        replication_driver_data=six.text_type(provider_location4))
+        replication_driver_data=str(provider_location4))
 
     test_snapshot = fake_snapshot.fake_snapshot_obj(
         context=ctx, id=snapshot_id,
         name='my_snap', size=2,
-        provider_location=six.text_type(snap_location),
+        provider_location=str(snap_location),
         host=fake_host, volume=test_volume)
 
     test_legacy_snapshot = fake_snapshot.fake_snapshot_obj(
         context=ctx, id=test_volume.id, name='my_snap', size=2,
-        provider_location=six.text_type(legacy_provider_location),
+        provider_location=str(legacy_provider_location),
         host=fake_host, volume=test_volume)
 
     test_failed_snap = fake_snapshot.fake_snapshot_obj(
@@ -279,13 +278,13 @@ class PowerMaxData(object):
         id='4732de9b-98a4-4b6d-ae4b-3cafb3d34220',
         name=failed_resource,
         size=2,
-        provider_location=six.text_type(snap_location),
+        provider_location=str(snap_location),
         host=fake_host, volume=test_volume)
 
     test_snapshot_manage = fake_snapshot.fake_snapshot_obj(
         context=ctx, id=snapshot_id,
         name='my_snap', size=2,
-        provider_location=six.text_type(snap_location),
+        provider_location=str(snap_location),
         host=fake_host, volume=test_volume_snap_manage,
         display_name='my_snap')
 
@@ -420,6 +419,13 @@ class PowerMaxData(object):
         'array': array,
         'interval': 3,
         'retries': 120}
+    extra_specs_no_pool_name = {
+        'slo': slo,
+        'workload': workload,
+        'srp': srp,
+        'array': array,
+        'interval': 3,
+        'retries': 120}
     extra_specs_optimized = {
         'pool_name': u'Optimized+None+SRP_1+000197800123',
         'slo': 'Optimized', 'workload': 'None',
@@ -451,6 +457,7 @@ class PowerMaxData(object):
     rep_extra_specs['sync_retries'] = 200
     rep_extra_specs['rdf_group_label'] = rdf_group_name_1
     rep_extra_specs['rdf_group_no'] = rdf_group_no_1
+    rep_extra_specs[utils.DISABLE_PROTECTED_SNAP] = False
     rep_extra_specs2 = deepcopy(rep_extra_specs)
     rep_extra_specs2[utils.PORTGROUPNAME] = port_group_name_f
     rep_extra_specs3 = deepcopy(rep_extra_specs)
@@ -540,7 +547,7 @@ class PowerMaxData(object):
         provider_auth=None, volume_type_ids=['abc'],
         group_type_id='grptypeid',
         volume_types=test_volume_type_list,
-        host=fake_host, provider_location=six.text_type(provider_location))
+        host=fake_host, provider_location=str(provider_location))
 
     test_group_failed = group.Group(
         context=None, name=failed_resource,
@@ -551,7 +558,7 @@ class PowerMaxData(object):
         provider_auth=None, volume_type_ids=['abc'],
         group_type_id='grptypeid',
         volume_types=test_volume_type_list,
-        host=fake_host, provider_location=six.text_type(provider_location),
+        host=fake_host, provider_location=str(provider_location),
         replication_status=fields.ReplicationStatus.DISABLED)
 
     test_rep_group = fake_group.fake_group_obj(
@@ -589,9 +596,9 @@ class PowerMaxData(object):
 
     test_volume_group_member = fake_volume.fake_volume_obj(
         context=ctx, name='vol1', size=2, provider_auth=None,
-        provider_location=six.text_type(provider_location),
+        provider_location=str(provider_location),
         volume_type=test_volume_type, host=fake_host,
-        replication_driver_data=six.text_type(provider_location3),
+        replication_driver_data=str(provider_location3),
         group_id=test_vol_grp_name_id_only)
 
     # masking view dict
@@ -944,6 +951,9 @@ class PowerMaxData(object):
     powermax_model_details = {'symmetrixId': array,
                               'model': 'PowerMax_2000',
                               'ucode': '5978.1091.1092'}
+    powermax_model_100 = {'symmetrixId': array,
+                          'model': 'PowerMax_2500',
+                          'microcode': '6079.65.0'}
     vmax_slo_details = {'sloId': ['Diamond', 'Optimized']}
     vmax_model_details = {'model': 'VMAX450F'}
 
@@ -1348,6 +1358,7 @@ class PowerMaxData(object):
     platform = 'Linux-4.4.0-104-generic-x86_64-with-Ubuntu-16.04-xenial'
     unisphere_version = u'V9.2.0.0'
     unisphere_version_90 = "V9.0.0.1"
+    unisphere_version_100 = "V10.0.0.0"
     openstack_release = '12.0.0.0b3.dev401'
     openstack_version = '12.0.0'
     python_version = '2.7.12'
@@ -1572,7 +1583,7 @@ class PowerMaxData(object):
 
     r2_sg_list = deepcopy(r1_sg_list)
     replication_model = (
-        {'provider_location': six.text_type(provider_location),
+        {'provider_location': str(provider_location),
          'metadata': {'DeviceID': device_id,
                       'DeviceLabel': 'OS-%s' % volume_id,
                       'ArrayID': array,
@@ -1591,7 +1602,7 @@ class PowerMaxData(object):
                       'R2-RDFG': rdf_group_no_1}})
 
     non_replication_model = (
-        {'provider_location': six.text_type(provider_location),
+        {'provider_location': str(provider_location),
          'metadata': {'DeviceID': device_id,
                       'DeviceLabel': 'OS-%s' % volume_id,
                       'ArrayID': array,
