@@ -165,11 +165,18 @@ class PowerMaxMaskingTest(test.TestCase):
         rest.PowerMaxRest, 'get_num_vols_in_sg', side_effect=[2, 1, 1])
     def test_move_volume_between_storage_groups(
             self, mock_num, mock_parent, mock_rm, mck_mod):
+        extra_specs = {'pool_name': u'Diamond+DSS+SRP_1+000197800123',
+                       'slo': 'Diamond',
+                       'workload': 'DSS',
+                       'srp': 'SRP_1',
+                       'array': '000197800123',
+                       'interval': 3,
+                       'retries': 120}
         for x in range(0, 3):
             self.driver.masking.move_volume_between_storage_groups(
                 self.data.array, self.data.device_id,
                 self.data.storagegroup_name_i, self.data.storagegroup_name_f,
-                self.data.extra_specs)
+                extra_specs)
         mock_rm.assert_called_once()
         ref_payload = (
             {"executionOption": "ASYNCHRONOUS",
@@ -862,20 +869,18 @@ class PowerMaxMaskingTest(test.TestCase):
                        '_delete_cascaded_storage_groups')
     @mock.patch.object(rest.PowerMaxRest, 'get_num_vols_in_sg',
                        side_effect=[1, 3])
-    @mock.patch.object(rest.PowerMaxRest, 'delete_storage_group')
     @mock.patch.object(masking.PowerMaxMasking, 'get_parent_sg_from_child',
                        side_effect=[None, 'parent_sg_name', 'parent_sg_name'])
     def test_last_vol_no_masking_views(
-            self, mock_get_parent, mock_delete, mock_num_vols,
+            self, mock_get_parent, mock_num_vols,
             mock_delete_casc, mock_remove):
         for x in range(0, 3):
             self.mask._last_vol_no_masking_views(
                 self.data.array, self.data.storagegroup_name_i,
                 self.device_id, self.volume_name, self.extra_specs,
                 False)
-        self.assertEqual(1, mock_delete.call_count)
         self.assertEqual(1, mock_delete_casc.call_count)
-        self.assertEqual(1, mock_remove.call_count)
+        self.assertEqual(2, mock_remove.call_count)
 
     @mock.patch.object(masking.PowerMaxMasking,
                        '_remove_last_vol_and_delete_sg')
@@ -1211,6 +1216,23 @@ class PowerMaxMaskingTest(test.TestCase):
             utils.PowerMaxUtils.truncate_string.assert_called_once_with(
                 'OptimizedNONE', 10)
 
+    @mock.patch.object(masking.PowerMaxMasking,
+                       '_clean_up_child_storage_group')
+    @mock.patch.object(masking.PowerMaxMasking,
+                       'move_volume_between_storage_groups')
+    @mock.patch.object(masking.PowerMaxMasking,
+                       '_return_volume_to_fast_managed_group')
+    def test_pre_multiattach_pool_no_pool_name(self, mock_return, mck_move,
+                                               mck_clean):
+        with mock.patch.object(utils.PowerMaxUtils, 'truncate_string',
+                               return_value='DiamondDSS'):
+            self.mask.pre_multiattach(
+                self.data.array, self.data.device_id,
+                self.data.masking_view_dict_multiattach,
+                self.data.extra_specs_no_pool_name)
+            utils.PowerMaxUtils.truncate_string.assert_called_once_with(
+                'DiamondDSS', 10)
+
     @mock.patch.object(
         rest.PowerMaxRest, 'get_storage_group_list',
         side_effect=[
