@@ -18,6 +18,7 @@
 """Tests for NetApp API layer"""
 
 from unittest import mock
+import urllib
 
 import ddt
 from lxml import etree
@@ -26,8 +27,6 @@ from oslo_utils import netutils
 import paramiko
 import requests
 from requests import auth
-import six
-from six.moves import urllib
 
 from cinder import exception
 from cinder.i18n import _
@@ -90,20 +89,15 @@ class NetAppApiServerTests(test.TestCase):
               {'params': {'major': '!*', 'minor': '20a'}})
     @ddt.unpack
     def test_set_api_version_value_error(self, params):
-        """Tests Value Error on setting non-integer version"""
+        """Tests Value Error on setting values incompatible with integer"""
         self.assertRaises(ValueError, self.root.set_api_version, **params)
 
     def test_set_api_version_valid(self):
-        """Tests Value Error on setting non-integer version"""
+        """Tests no Error on setting values compatible with integer"""
         args = {'major': '20', 'minor': 1}
 
-        expected_call_args_list = [mock.call('20'), mock.call(1)]
-
-        mock_invoke = self.mock_object(six, 'text_type', return_value='str')
         self.root.set_api_version(**args)
 
-        self.assertEqual(expected_call_args_list, mock_invoke.call_args_list)
-
     @ddt.data({'params': {'result': zapi_fakes.FAKE_RESULT_API_ERR_REASON}},
               {'params': {'result': zapi_fakes.FAKE_RESULT_API_ERRNO_INVALID}},
               {'params': {'result': zapi_fakes.FAKE_RESULT_API_ERRNO_VALID}})
@@ -174,15 +168,23 @@ class NetAppApiServerTests(test.TestCase):
 
         mock_invoke.assert_called_with(zapi_fakes.FAKE_XML_STR)
 
-    def test__build_opener_not_implemented_error(self):
-        """Tests whether certificate style authorization raises Exception"""
-        self.root._auth_style = 'not_basic_auth'
+    def test_build_opener_with_certificate_auth(self):
+        """Tests whether build opener works with """
+        """valid certificate parameters"""
+        self.root._private_key_file = 'fake_key.pem'
+        self.root._certificate_file = 'fake_cert.pem'
+        auth_handler = self.mock_object(self.root,
+                                        '_create_certificate_auth_handler',
+                                        mock.Mock(return_value='fake_auth'))
+        expected_opener = 'fake_auth'
+        self.mock_object(urllib.request, 'build_opener', auth_handler)
+        self.root._build_opener()
+        self.assertEqual(self.root._opener, expected_opener)
+        self.root._create_certificate_auth_handler.assert_called()
 
-        self.assertRaises(NotImplementedError, self.root._build_opener)
-
-    def test__build_opener_valid(self):
-        """Tests whether build opener works with valid parameters"""
-        self.root._auth_style = 'basic_auth'
+    def test__build_opener_default(self):
+        """Tests whether build opener works with """
+        """default(basic auth) parameters"""
         mock_invoke = self.mock_object(urllib.request, 'build_opener')
 
         self.root._build_opener()
@@ -843,7 +845,9 @@ class NetAppRestApiServerTests(test.TestCase):
 
         self.assertEqual(expected_vserver, res)
 
-    def test__build_session(self):
+    def test__build_session_with_basic_auth(self):
+        """Tests whether build session works with """
+        """default(basic auth) parameters"""
         fake_session = mock.Mock()
         mock_requests_session = self.mock_object(
             requests, 'Session', mock.Mock(return_value=fake_session))
@@ -862,6 +866,28 @@ class NetAppRestApiServerTests(test.TestCase):
         mock_requests_session.assert_called_once_with()
         mock_auth.assert_called_once_with()
 
+    def test__build_session_with_certificate_auth(self):
+        """Tests whether build session works with """
+        """valid certificate parameters"""
+        self.rest_client._private_key_file = 'fake_key.pem'
+        self.rest_client._certificate_file = 'fake_cert.pem'
+        self.rest_client._certificate_host_validation = False
+        fake_session = mock.Mock()
+        mock_requests_session = self.mock_object(
+            requests, 'Session', mock.Mock(return_value=fake_session))
+        mock_auth = self.mock_object(
+            self.rest_client, '_create_certificate_auth_handler',
+            mock.Mock(return_value=('fake_cert', 'fake_verify')))
+        self.rest_client._build_session(zapi_fakes.FAKE_HEADERS)
+        self.assertEqual(fake_session, self.rest_client._session)
+        self.assertEqual(('fake_cert', 'fake_verify'),
+                         (self.rest_client._session.cert,
+                          self.rest_client._session.verify))
+        self.assertEqual(zapi_fakes.FAKE_HEADERS,
+                         self.rest_client._session.headers)
+        mock_requests_session.assert_called_once_with()
+        mock_auth.assert_called_once_with()
+
     @ddt.data(True, False)
     def test__build_headers(self, enable_tunneling):
         self.rest_client._vserver = zapi_fakes.VSERVER_NAME
@@ -886,3 +912,33 @@ class NetAppRestApiServerTests(test.TestCase):
 
         expected = auth.HTTPBasicAuth(username, password)
         self.assertEqual(expected.__dict__, res.__dict__)
+
+    def test__create_certificate_auth_handler_default(self):
+        """Test whether create certificate auth handler """
+        """works with default params"""
+        self.rest_client._private_key_file = 'fake_key.pem'
+        self.rest_client._certificate_file = 'fake_cert.pem'
+        self.rest_client._certificate_host_validation = False
+        cert = self.rest_client._certificate_file, \
+            self.rest_client._private_key_file
+        self.rest_client._session = mock.Mock()
+        if not self.rest_client._certificate_host_validation:
+            self.assertFalse(self.rest_client._certificate_host_validation)
+        res = self.rest_client._create_certificate_auth_handler()
+        self.assertEqual(res,
+                         (cert, self.rest_client._certificate_host_validation))
+
+    def test__create_certificate_auth_handler_with_host_validation(self):
+        """Test whether create certificate auth handler """
+        """works with host validation enabled"""
+        self.rest_client._private_key_file = 'fake_key.pem'
+        self.rest_client._certificate_file = 'fake_cert.pem'
+        self.rest_client._ca_certificate_file = 'fake_ca_cert.crt'
+        self.rest_client._certificate_host_validation = True
+        cert = self.rest_client._certificate_file, \
+            self.rest_client._private_key_file
+        self.rest_client._session = mock.Mock()
+        if self.rest_client._certificate_host_validation:
+            self.assertTrue(self.rest_client._certificate_host_validation)
+        res = self.rest_client._create_certificate_auth_handler()
+        self.assertEqual(res, (cert, self.rest_client._ca_certificate_file))
