@@ -16,7 +16,6 @@
 """Unit tests for the NetApp NFS storage driver."""
 import copy
 import os
-import threading
 import time
 from unittest import mock
 
@@ -129,6 +128,7 @@ class NetAppNfsDriverTestCase(test.TestCase):
               {'replication_status': fields.ReplicationStatus.ENABLED})
     def test_create_volume(self, model_update):
         self.mock_object(self.driver, '_ensure_shares_mounted')
+        self.mock_object(self.driver, '_ensure_flexgroup_not_in_cg')
         self.mock_object(na_utils, 'get_volume_extra_specs')
         self.mock_object(self.driver, '_do_create_volume')
         self.mock_object(self.driver, '_do_qos_for_volume')
@@ -146,6 +146,7 @@ class NetAppNfsDriverTestCase(test.TestCase):
         volume = copy.deepcopy(fake.NFS_VOLUME)
         volume['host'] = '%s@%s' % (fake.HOST_NAME, fake.BACKEND_NAME)
         self.mock_object(self.driver, '_ensure_shares_mounted')
+        self.mock_object(self.driver, '_ensure_flexgroup_not_in_cg')
 
         self.assertRaises(exception.InvalidHost,
                           self.driver.create_volume,
@@ -153,6 +154,7 @@ class NetAppNfsDriverTestCase(test.TestCase):
 
     def test_create_volume_exception(self):
         self.mock_object(self.driver, '_ensure_shares_mounted')
+        self.mock_object(self.driver, '_ensure_flexgroup_not_in_cg')
         self.mock_object(na_utils, 'get_volume_extra_specs')
         mock_create = self.mock_object(self.driver, '_do_create_volume')
         mock_create.side_effect = Exception
@@ -263,7 +265,9 @@ class NetAppNfsDriverTestCase(test.TestCase):
                           fake.CLONE_SOURCE,
                           fake.NFS_VOLUME)
 
-    def test_create_volume_from_snapshot(self):
+    @ddt.data(True, False)
+    def test_create_volume_from_snapshot(self, is_flexgroup):
+        provider_location = fake.POOL_NAME
         volume = fake.VOLUME
         expected_source = {
             'name': fake.SNAPSHOT_NAME,
@@ -272,22 +276,90 @@ class NetAppNfsDriverTestCase(test.TestCase):
         }
         mock_clone_call = self.mock_object(
             self.driver, '_clone_source_to_destination_volume',
-            return_value='fake')
+            return_value=provider_location)
+        self.mock_object(self.driver, '_ensure_flexgroup_not_in_cg')
+        self.mock_object(self.driver, '_is_flexgroup',
+                         return_value=is_flexgroup)
+        self.mock_object(self.driver, '_is_flexgroup_clone_file_supported',
+                         return_value=not is_flexgroup)
+        mock_super_create = self.mock_object(
+            nfs.NfsDriver, 'create_volume_from_snapshot',
+            return_value=provider_location)
+        mock_do_qos = self.mock_object(
+            self.driver, '_do_qos_for_file_flexgroup',
+            return_value=provider_location)
 
         retval = self.driver.create_volume_from_snapshot(volume, fake.SNAPSHOT)
 
-        self.assertEqual('fake', retval)
-        mock_clone_call.assert_called_once_with(expected_source, volume)
+        self.assertEqual(provider_location, retval)
+        if is_flexgroup:
+            mock_clone_call.assert_not_called()
+            mock_super_create.assert_called_once_with(volume, fake.SNAPSHOT)
+            mock_do_qos.assert_called_once_with(volume, provider_location)
+        else:
+            mock_clone_call.assert_called_once_with(expected_source, volume)
+            mock_do_qos.assert_not_called()
+            mock_super_create.assert_not_called()
 
-    def test_create_cloned_volume(self):
+    @ddt.data(True, False)
+    def test_create_cloned_volume(self, is_flexgroup):
         provider_location = fake.POOL_NAME
+        volume = fake.VOLUME
         src_vref = fake.CLONE_SOURCE
-        self.mock_object(self.driver, '_clone_source_to_destination_volume',
-                         return_value=provider_location)
+        mock_clone_call = self.mock_object(
+            self.driver, '_clone_source_to_destination_volume',
+            return_value=provider_location)
+        self.mock_object(self.driver, '_ensure_flexgroup_not_in_cg')
+        self.mock_object(self.driver, '_is_flexgroup',
+                         return_value=is_flexgroup)
+        self.mock_object(self.driver, '_is_flexgroup_clone_file_supported',
+                         return_value=not is_flexgroup)
+        mock_super_create = self.mock_object(
+            nfs.NfsDriver, 'create_cloned_volume',
+            return_value=provider_location)
+        mock_do_qos = self.mock_object(
+            self.driver, '_do_qos_for_file_flexgroup',
+            return_value=provider_location)
+
+        result = self.driver.create_cloned_volume(volume, src_vref)
 
-        result = self.driver.create_cloned_volume(fake.NFS_VOLUME,
-                                                  src_vref)
         self.assertEqual(provider_location, result)
+        if is_flexgroup:
+            mock_clone_call.assert_not_called()
+            mock_super_create.assert_called_once_with(volume, src_vref)
+            mock_do_qos.assert_called_once_with(volume, provider_location)
+        else:
+            mock_clone_call.assert_called_once_with(src_vref, volume)
+            mock_do_qos.assert_not_called()
+            mock_super_create.assert_not_called()
+
+    def test_do_qos_for_file_flexgroup(self):
+        volume = {'provider_location': 'fake'}
+        extra_specs = 'fake_extra'
+        model = {'provider_location': 'fake'}
+        vol_model = {'replication': 'fake'}
+        expected_model = {
+            'replication': vol_model['replication'],
+            'provider_location': model['provider_location'],
+        }
+        self.mock_object(self.driver, '_do_qos_for_volume')
+        self.mock_object(self.driver, '_get_volume_model_update',
+                         return_value=vol_model)
+        mock_extra = self.mock_object(na_utils, 'get_volume_extra_specs',
+                                      return_value=extra_specs)
+
+        model_updated = self.driver._do_qos_for_file_flexgroup(volume, model)
+
+        self.assertEqual(model_updated, expected_model)
+        mock_extra.assert_called_once_with(volume)
+
+    def test_do_qos_for_file_flexgroup_error(self):
+        self.mock_object(na_utils, 'get_volume_extra_specs',
+                         side_effect=exception.NotFound)
+
+        self.assertRaises(exception.VolumeBackendAPIException,
+                          self.driver._do_qos_for_file_flexgroup,
+                          fake.VOLUME, 'fake_model')
 
     def test_do_qos_for_volume(self):
         self.assertRaises(NotImplementedError,
@@ -295,18 +367,83 @@ class NetAppNfsDriverTestCase(test.TestCase):
                           fake.NFS_VOLUME,
                           fake.EXTRA_SPECS)
 
-    def test_create_snapshot(self):
-
+    @ddt.data(True, False)
+    def test_create_snapshot(self, is_flexgroup):
+        self.mock_object(self.driver, '_is_flexgroup',
+                         return_value=is_flexgroup)
+        self.mock_object(self.driver, '_is_flexgroup_clone_file_supported',
+                         return_value=not is_flexgroup)
         mock_clone_backing_file_for_volume = self.mock_object(
             self.driver, '_clone_backing_file_for_volume')
+        mock_snap_flexgroup = self.mock_object(
+            self.driver, '_create_snapshot_for_flexgroup')
 
         self.driver.create_snapshot(fake.SNAPSHOT)
 
-        mock_clone_backing_file_for_volume.assert_called_once_with(
-            fake.SNAPSHOT['volume_name'], fake.SNAPSHOT['name'],
-            fake.SNAPSHOT['volume_id'], is_snapshot=True)
+        if is_flexgroup:
+            mock_snap_flexgroup.assert_called_once_with(fake.SNAPSHOT)
+            mock_clone_backing_file_for_volume.assert_not_called()
+        else:
+            mock_snap_flexgroup.assert_not_called()
+            mock_clone_backing_file_for_volume.assert_called_once_with(
+                fake.SNAPSHOT['volume_name'], fake.SNAPSHOT['name'],
+                fake.SNAPSHOT['volume_id'], is_snapshot=True)
 
-    def test_delete_snapshot(self):
+    def test_create_snapshot_for_flexgroup(self):
+        source_vol = {
+            'id': fake.SNAPSHOT['volume_id'],
+            'name': fake.SNAPSHOT['volume_name'],
+            'volume_type_id': fake.SNAPSHOT['volume_type_id'],
+        }
+        snap_vol = {
+            'name': '%s.%s' % (fake.SNAPSHOT['volume_name'],
+                               fake.SNAPSHOT['id']),
+            'host': fake.HOST_NAME,
+        }
+        mock_super_snapshot = self.mock_object(nfs.NfsDriver,
+                                               'create_snapshot')
+        mock_extra_specs = self.mock_object(na_utils,
+                                            'get_volume_extra_specs')
+        mock_extra_specs.return_value = fake.EXTRA_SPECS
+        mock_get_info = self.mock_object(na_utils,
+                                         'get_valid_qos_policy_group_info')
+        mock_get_info.return_value = fake.QOS_POLICY_GROUP_INFO
+        mock_get_host = self.mock_object(self.driver,
+                                         '_get_volume_host')
+        mock_get_host.return_value = fake.HOST_NAME
+        mock_set_policy = self.mock_object(self.driver,
+                                           '_set_qos_policy_group_on_volume')
+
+        self.driver._create_snapshot_for_flexgroup(fake.SNAPSHOT)
+
+        mock_super_snapshot.assert_has_calls([
+            mock.call(fake.SNAPSHOT)])
+        mock_get_host.assert_has_calls([
+            mock.call(source_vol['id'])])
+        mock_extra_specs.assert_has_calls([
+            mock.call(source_vol)])
+        mock_get_info.assert_has_calls([
+            mock.call(source_vol, fake.EXTRA_SPECS)])
+        mock_set_policy.assert_has_calls([
+            mock.call(snap_vol, fake.QOS_POLICY_GROUP_INFO, False)])
+
+    def test_create_snapshot_for_flexgroup_error(self):
+        self.mock_object(nfs.NfsDriver, 'create_snapshot',
+                         side_effect=exception.NotFound)
+
+        self.assertRaises(exception.VolumeBackendAPIException,
+                          self.driver._create_snapshot_for_flexgroup,
+                          fake.SNAPSHOT)
+
+    def test_set_qos_policy_group_on_volume(self):
+        self.assertRaises(NotImplementedError,
+                          self.driver._set_qos_policy_group_on_volume,
+                          fake.NFS_VOLUME,
+                          fake.QOS_POLICY_GROUP_INFO,
+                          False)
+
+    @ddt.data(True, False)
+    def test_delete_snapshot(self, is_flexgroup):
         updates = {
             'name': fake.SNAPSHOT_NAME,
             'volume_size': fake.SIZE,
@@ -316,11 +453,22 @@ class NetAppNfsDriverTestCase(test.TestCase):
         }
         snapshot = fake_snapshot.fake_snapshot_obj(self.ctxt, **updates)
         self.mock_object(self.driver, '_delete_file')
+        self.mock_object(self.driver, '_is_flexgroup',
+                         return_value=is_flexgroup)
+        self.mock_object(self.driver, '_is_flexgroup_clone_file_supported',
+                         return_value=not is_flexgroup)
+        mock_super_delete = self.mock_object(nfs.NfsDriver,
+                                             'delete_snapshot')
 
         self.driver.delete_snapshot(snapshot)
 
-        self.driver._delete_file.assert_called_once_with(snapshot.volume_id,
-                                                         snapshot.name)
+        if is_flexgroup:
+            mock_super_delete.assert_called_once_with(snapshot)
+            self.driver._delete_file.assert_not_called()
+        else:
+            mock_super_delete.assert_not_called()
+            self.driver._delete_file.assert_called_once_with(
+                snapshot.volume_id, snapshot.name)
 
     @ddt.data(fake.NFS_SHARE, fake.NFS_SHARE_IPV6)
     def test__get_volume_location(self, provider):
@@ -339,6 +487,10 @@ class NetAppNfsDriverTestCase(test.TestCase):
                           fake.VOLUME_NAME, fake.CLONE_SOURCE_NAME,
                           fake.VOLUME_ID, share=None)
 
+    def test__is_flexgroup(self):
+        self.assertRaises(NotImplementedError,
+                          self.driver._is_flexgroup)
+
     def test__get_provider_location(self):
         updates = {'provider_location': fake.PROVIDER_LOCATION}
         volume = fake_volume.fake_volume_obj(self.ctxt, **updates)
@@ -348,6 +500,15 @@ class NetAppNfsDriverTestCase(test.TestCase):
 
         self.assertEqual(fake.PROVIDER_LOCATION, retval)
 
+    def test__get_volume_host(self):
+        updates = {'host': fake.HOST_NAME}
+        volume = fake_volume.fake_volume_obj(self.ctxt, **updates)
+        self.mock_object(self.driver.db, 'volume_get', return_value=volume)
+
+        retval = self.driver._get_volume_host(fake.VOLUME_ID)
+
+        self.assertEqual(fake.HOST_NAME, retval)
+
     @ddt.data(None, processutils.ProcessExecutionError)
     def test__volume_not_present(self, side_effect):
         self.mock_object(self.driver, '_get_volume_path')
@@ -382,6 +543,7 @@ class NetAppNfsDriverTestCase(test.TestCase):
 
     def test_copy_image_to_volume_base_exception(self):
         mock_info_log = self.mock_object(nfs_base.LOG, 'info')
+        self.mock_object(self.driver, '_ensure_flexgroup_not_in_cg')
         self.mock_object(remotefs.RemoteFSDriver, 'copy_image_to_volume',
                          side_effect=exception.NfsException)
 
@@ -393,6 +555,11 @@ class NetAppNfsDriverTestCase(test.TestCase):
 
     def test_copy_image_to_volume(self):
         mock_log = self.mock_object(nfs_base, 'LOG')
+        self.mock_object(self.driver, '_is_flexgroup',
+                         return_value=False)
+        self.mock_object(self.driver, '_is_flexgroup_clone_file_supported',
+                         return_value=True)
+        self.mock_object(self.driver, '_ensure_flexgroup_not_in_cg')
         mock_copy_image = self.mock_object(
             remotefs.RemoteFSDriver, 'copy_image_to_volume')
         mock_register_image = self.mock_object(
@@ -405,7 +572,7 @@ class NetAppNfsDriverTestCase(test.TestCase):
 
         mock_copy_image.assert_called_once_with(
             'fake_context', fake.NFS_VOLUME, 'fake_img_service',
-            fake.IMAGE_FILE_ID)
+            fake.IMAGE_FILE_ID, disable_sparse=False)
         self.assertEqual(1, mock_log.info.call_count)
         mock_register_image.assert_called_once_with(
             fake.NFS_VOLUME, fake.IMAGE_FILE_ID)
@@ -452,36 +619,6 @@ class NetAppNfsDriverTestCase(test.TestCase):
         os.path.exists.assert_called_once_with(
             'dir/' + fake.CLONE_DESTINATION_NAME)
 
-    def test__spawn_clean_cache_job_clean_job_setup(self):
-        self.driver.cleaning = True
-        mock_debug_log = self.mock_object(nfs_base.LOG, 'debug')
-        self.mock_object(utils, 'synchronized', return_value=lambda f: f)
-
-        retval = self.driver._spawn_clean_cache_job()
-
-        self.assertIsNone(retval)
-        self.assertEqual(1, mock_debug_log.call_count)
-
-    def test__spawn_clean_cache_job_new_clean_job(self):
-
-        class FakeTimer(object):
-            def start(self):
-                pass
-
-        fake_timer = FakeTimer()
-        self.mock_object(utils, 'synchronized', return_value=lambda f: f)
-        self.mock_object(fake_timer, 'start')
-        self.mock_object(nfs_base.LOG, 'debug')
-        self.mock_object(self.driver, '_clean_image_cache')
-        self.mock_object(threading, 'Timer', return_value=fake_timer)
-
-        retval = self.driver._spawn_clean_cache_job()
-
-        self.assertIsNone(retval)
-        threading.Timer.assert_called_once_with(
-            0, self.driver._clean_image_cache)
-        fake_timer.start.assert_called_once_with()
-
     def test_cleanup_volume_on_failure(self):
         path = '%s/%s' % (fake.NFS_SHARE, fake.NFS_VOLUME['name'])
         mock_local_path = self.mock_object(self.driver, 'local_path')
@@ -561,10 +698,16 @@ class NetAppNfsDriverTestCase(test.TestCase):
 
         self.assertIn("nfs://host/path/image-id", locations)
 
-    def test_extend_volume(self):
+    @ddt.data(None, 'raw', 'qcow2')
+    @mock.patch('cinder.objects.volume.Volume.get_by_id')
+    def test_extend_volume(self, file_format, mock_get):
 
+        volume = fake_volume.fake_volume_obj(self.ctxt)
+        if file_format:
+            volume.admin_metadata = {'format': file_format}
+        mock_get.return_value = volume
         new_size = 100
-        volume_copy = copy.copy(fake.VOLUME)
+        volume_copy = copy.copy(volume)
         volume_copy['size'] = new_size
 
         path = '%s/%s' % (fake.NFS_SHARE, fake.NFS_VOLUME['name'])
@@ -578,18 +721,22 @@ class NetAppNfsDriverTestCase(test.TestCase):
         mock_do_qos_for_volume = self.mock_object(self.driver,
                                                   '_do_qos_for_volume')
 
-        self.driver.extend_volume(fake.VOLUME, new_size)
+        self.driver.extend_volume(volume, new_size)
 
-        mock_resize_image_file.assert_called_once_with(path, new_size)
-        mock_get_volume_extra_specs.assert_called_once_with(fake.VOLUME)
+        mock_resize_image_file.assert_called_once_with(path, new_size,
+                                                       file_format=file_format)
+        mock_get_volume_extra_specs.assert_called_once_with(volume)
         mock_do_qos_for_volume.assert_called_once_with(volume_copy,
                                                        fake.EXTRA_SPECS,
                                                        cleanup=False)
 
-    def test_extend_volume_resize_error(self):
+    @mock.patch('cinder.objects.volume.Volume.get_by_id')
+    def test_extend_volume_resize_error(self, mock_get):
 
+        volume = fake_volume.fake_volume_obj(self.ctxt)
+        mock_get.return_value = volume
         new_size = 100
-        volume_copy = copy.copy(fake.VOLUME)
+        volume_copy = copy.copy(volume)
         volume_copy['size'] = new_size
 
         path = '%s/%s' % (fake.NFS_SHARE, fake.NFS_VOLUME['name'])
@@ -606,17 +753,21 @@ class NetAppNfsDriverTestCase(test.TestCase):
 
         self.assertRaises(exception.VolumeBackendAPIException,
                           self.driver.extend_volume,
-                          fake.VOLUME,
+                          volume,
                           new_size)
 
-        mock_resize_image_file.assert_called_once_with(path, new_size)
+        mock_resize_image_file.assert_called_once_with(path, new_size,
+                                                       file_format=None)
         self.assertFalse(mock_get_volume_extra_specs.called)
         self.assertFalse(mock_do_qos_for_volume.called)
 
-    def test_extend_volume_qos_error(self):
+    @mock.patch('cinder.objects.volume.Volume.get_by_id')
+    def test_extend_volume_qos_error(self, mock_get):
 
+        volume = fake_volume.fake_volume_obj(self.ctxt)
+        mock_get.return_value = volume
         new_size = 100
-        volume_copy = copy.copy(fake.VOLUME)
+        volume_copy = copy.copy(volume)
         volume_copy['size'] = new_size
 
         path = '%s/%s' % (fake.NFS_SHARE, fake.NFS_VOLUME['name'])
@@ -634,11 +785,12 @@ class NetAppNfsDriverTestCase(test.TestCase):
 
         self.assertRaises(exception.VolumeBackendAPIException,
                           self.driver.extend_volume,
-                          fake.VOLUME,
+                          volume,
                           new_size)
 
-        mock_resize_image_file.assert_called_once_with(path, new_size)
-        mock_get_volume_extra_specs.assert_called_once_with(fake.VOLUME)
+        mock_resize_image_file.assert_called_once_with(path, new_size,
+                                                       file_format=None)
+        mock_get_volume_extra_specs.assert_called_once_with(volume)
         mock_do_qos_for_volume.assert_called_once_with(volume_copy,
                                                        fake.EXTRA_SPECS,
                                                        cleanup=False)
@@ -742,7 +894,7 @@ class NetAppNfsDriverTestCase(test.TestCase):
             self.assertEqual(model_update['replication_status'],
                              actual_model_update['replication_status'])
         else:
-            self.assertFalse('replication_status' in actual_model_update)
+            self.assertNotIn('replication_status', actual_model_update)
         self.driver._check_volume_type.assert_called_once_with(
             volume, self.fake_nfs_export_1, test_file, {})
 
@@ -894,13 +1046,22 @@ class NetAppNfsDriverTestCase(test.TestCase):
             self.driver, '_delete_snapshots_marked_for_deletion')
         mock_call_ems_logging = self.mock_object(
             self.driver, '_handle_ems_logging')
+        mock_call_clean_image_cache = self.mock_object(
+            self.driver, '_clean_image_cache')
+
+        # image cache cleanup task can be configured with custom timeout
+        cache_cleanup_interval = loopingcalls.ONE_HOUR
+        self.driver.configuration.netapp_nfs_image_cache_cleanup_interval = (
+            cache_cleanup_interval)
 
         self.driver._add_looping_tasks()
 
         mock_add_task.assert_has_calls([
             mock.call(mock_call_snap_cleanup, loopingcalls.ONE_MINUTE,
                       loopingcalls.ONE_MINUTE),
-            mock.call(mock_call_ems_logging, loopingcalls.ONE_HOUR)])
+            mock.call(mock_call_ems_logging, loopingcalls.ONE_HOUR),
+            mock.call(mock_call_clean_image_cache, cache_cleanup_interval)
+        ])
 
     def test__clone_from_cache(self):
         image_id = 'fake_image_id'
@@ -941,8 +1102,81 @@ class NetAppNfsDriverTestCase(test.TestCase):
         mock_call__is_share_clone_compatible.assert_not_called()
         mock_call__do_clone_rel_img_cache.assert_not_called()
 
+    def test__find_share(self):
+        mock_extract = self.mock_object(volume_utils, 'extract_host',
+                                        return_value=fake.POOL_NAME)
+
+        pool_name = self.driver._find_share(fake.VOLUME)
+
+        self.assertEqual(pool_name, fake.POOL_NAME)
+        mock_extract.assert_called_once_with(fake.VOLUME['host'],
+                                             level='pool')
+
+    def test__find_share_error(self):
+        mock_extract = self.mock_object(volume_utils, 'extract_host',
+                                        return_value=None)
+
+        self.assertRaises(exception.InvalidHost,
+                          self.driver._find_share,
+                          fake.VOLUME)
+        mock_extract.assert_called_once_with(fake.VOLUME['host'],
+                                             level='pool')
+
+    def test__ensure_flexgroup_not_in_cg_raises(self):
+        self.mock_object(self.driver, '_is_flexgroup',
+                         return_value=True)
+        self.mock_object(volume_utils, 'is_group_a_cg_snapshot_type',
+                         return_value=True)
+        fake_v1 = {
+            'group': 'fake_group',
+            'host': 'fake_host',
+            'id': 'fake_id'
+        }
+        self.assertRaises(na_utils.NetAppDriverException,
+                          self.driver._ensure_flexgroup_not_in_cg,
+                          fake_v1)
+
+    def test__is_flexgroup_clone_file_supported(self):
+        self.assertRaises(NotImplementedError,
+                          self.driver._is_flexgroup_clone_file_supported)
+
     def test_update_migrated_volume(self):
         self.assertRaises(NotImplementedError,
                           self.driver.update_migrated_volume, self.ctxt,
                           fake.test_volume, mock.sentinel.new_volume,
                           mock.sentinel.original_status)
+
+    @ddt.data({'is_flexgroup': False, 'is_flexgroup_supported': False},
+              {'is_flexgroup': False, 'is_flexgroup_supported': True},
+              {'is_flexgroup': True, 'is_flexgroup_supported': False},
+              {'is_flexgroup': True, 'is_flexgroup_supported': True})
+    @ddt.unpack
+    def test_revert_to_snapshot(self, is_flexgroup, is_flexgroup_supported):
+        context = {}
+
+        self.mock_object(self.driver,
+                         '_is_flexgroup',
+                         return_value=is_flexgroup)
+        self.mock_object(self.driver,
+                         '_is_flexgroup_clone_file_supported',
+                         return_value=is_flexgroup_supported)
+        mock_revert_to_snapshot = self.mock_object(
+            remotefs.RemoteFSSnapDriverDistributed, 'revert_to_snapshot')
+        mock__revert_to_snapshot = self.mock_object(self.driver,
+                                                    '_revert_to_snapshot')
+
+        self.driver.revert_to_snapshot(context, fake.SNAPSHOT_VOLUME,
+                                       fake.SNAPSHOT)
+
+        if is_flexgroup and not is_flexgroup_supported:
+            mock_revert_to_snapshot.assert_called_once_with(
+                context, fake.SNAPSHOT_VOLUME, fake.SNAPSHOT)
+            mock__revert_to_snapshot.assert_not_called()
+        else:
+            mock__revert_to_snapshot.assert_called_once_with(
+                fake.SNAPSHOT_VOLUME, fake.SNAPSHOT)
+            mock_revert_to_snapshot.assert_not_called()
+
+    def test__revert_to_snapshot(self):
+        self.assertRaises(NotImplementedError, self.driver._revert_to_snapshot,
+                          fake.SNAPSHOT_VOLUME, fake.SNAPSHOT)
