@@ -13,9 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 from unittest import mock
 
-from oslo_config import cfg
 from oslo_utils import timeutils
 
 from cinder import exception as cinder_exception
@@ -23,8 +23,6 @@ from cinder.tests.unit import test
 from cinder.volume import configuration as conf
 from cinder.volume.drivers import linstordrv as drv
 
-CONF = cfg.CONF
-
 CINDER_UNKNOWN = 'unknown'
 DISKLESS = 'DISKLESS'
 LVM = 'LVM'
@@ -675,7 +673,6 @@ class LinstorBaseDriverTestCase(test.TestCase):
         m_rd_delete.assert_called_once()
 
     @mock.patch(DRIVER + 'LinstorBaseDriver._get_api_storage_pool_list')
-    @mock.patch(DRIVER + 'LinstorBaseDriver._api_volume_dfn_set_sp')
     @mock.patch(DRIVER + 'LinstorBaseDriver._get_api_volume_extend')
     @mock.patch(DRIVER + 'LinstorBaseDriver._api_rsc_create')
     @mock.patch(DRIVER + 'LinstorBaseDriver._api_snapshot_resource_restore')
@@ -691,7 +688,6 @@ class LinstorBaseDriverTestCase(test.TestCase):
                                          m_snap_rsc_restore,
                                          m_rsc_create,
                                          m_vol_extend,
-                                         m_vol_dfn,
                                          m_sp_list):
         m_rsc_dfn_create.return_value = True
         m_api_reply.return_value = True
@@ -701,7 +697,6 @@ class LinstorBaseDriverTestCase(test.TestCase):
         m_snap_rsc_restore.return_value = True
         m_rsc_create.return_value = True
         m_vol_extend.return_value = True
-        m_vol_dfn.return_value = True
         m_sp_list.return_value = (
             self._fake_driver.fake_api_storage_pool_list())
 
@@ -710,7 +705,6 @@ class LinstorBaseDriverTestCase(test.TestCase):
             CINDER_VOLUME, SNAPSHOT))
 
     @mock.patch(DRIVER + 'LinstorBaseDriver._get_api_storage_pool_list')
-    @mock.patch(DRIVER + 'LinstorBaseDriver._api_volume_dfn_set_sp')
     @mock.patch(DRIVER + 'LinstorBaseDriver._get_api_volume_extend')
     @mock.patch(DRIVER + 'LinstorBaseDriver._api_rsc_create')
     @mock.patch(DRIVER + 'LinstorBaseDriver._api_snapshot_resource_restore')
@@ -726,7 +720,6 @@ class LinstorBaseDriverTestCase(test.TestCase):
                                                       m_snap_rsc_restore,
                                                       m_rsc_create,
                                                       m_vol_extend,
-                                                      m_vol_dfn,
                                                       m_sp_list):
         m_rsc_dfn_create.return_value = True
         m_api_reply.return_value = True
@@ -736,7 +729,6 @@ class LinstorBaseDriverTestCase(test.TestCase):
         m_snap_rsc_restore.return_value = False
         m_rsc_create.return_value = True
         m_vol_extend.return_value = True
-        m_vol_dfn.return_value = True
         m_sp_list.return_value = (
             self._fake_driver.fake_api_storage_pool_list())
 
@@ -747,7 +739,6 @@ class LinstorBaseDriverTestCase(test.TestCase):
 
     @mock.patch(DRIVER + 'LinstorBaseDriver.delete_volume')
     @mock.patch(DRIVER + 'LinstorBaseDriver._get_api_storage_pool_list')
-    @mock.patch(DRIVER + 'LinstorBaseDriver._api_volume_dfn_set_sp')
     @mock.patch(DRIVER + 'LinstorBaseDriver._get_api_volume_extend')
     @mock.patch(DRIVER + 'LinstorBaseDriver._api_rsc_create')
     @mock.patch(DRIVER + 'LinstorBaseDriver._api_snapshot_resource_restore')
@@ -763,7 +754,6 @@ class LinstorBaseDriverTestCase(test.TestCase):
                                                      m_snap_rsc_restore,
                                                      m_rsc_create,
                                                      m_vol_extend,
-                                                     m_vol_dfn,
                                                      m_sp_list,
                                                      m_delete_volume):
         m_rsc_dfn_create.return_value = True
@@ -774,7 +764,6 @@ class LinstorBaseDriverTestCase(test.TestCase):
         m_snap_rsc_restore.return_value = True
         m_rsc_create.return_value = True
         m_vol_extend.return_value = True
-        m_vol_dfn.return_value = True
         m_sp_list.return_value = (
             self._fake_driver.fake_api_storage_pool_list())
         m_delete_volume.return_value = True
@@ -1122,13 +1111,15 @@ class LinstorIscsiDriverTestCase(test.TestCase):
     @mock.patch(DRIVER + 'LinstorIscsiDriver._get_volume_stats')
     def test_iscsi_get_volume_stats(self, m_vol_stats, m_rsc_list):
 
-        m_vol_stats.return_value = VOLUME_STATS_RESP
+        m_vol_stats.return_value = copy.deepcopy(VOLUME_STATS_RESP)
         m_rsc_list.return_value = RESOURCE_LIST
 
         val = self.driver.get_volume_stats()
 
-        expected = VOLUME_STATS_RESP
+        expected = copy.deepcopy(VOLUME_STATS_RESP)
         expected["storage_protocol"] = 'iSCSI'
+        expected["pools"][0]['location_info'] = (
+            'LinstorIscsiDriver:' + expected["pools"][0]['location_info'])
         self.assertEqual(expected, val)
 
     @mock.patch(DRIVER + 'linstor')
@@ -1190,11 +1181,13 @@ class LinstorDrbdDriverTestCase(test.TestCase):
 
     @mock.patch(DRIVER + 'LinstorDrbdDriver._get_volume_stats')
     def test_drbd_get_volume_stats(self, m_vol_stats):
-        m_vol_stats.return_value = VOLUME_STATS_RESP
+        m_vol_stats.return_value = copy.deepcopy(VOLUME_STATS_RESP)
 
         val = self.driver.get_volume_stats()
-        expected = VOLUME_STATS_RESP
+        expected = copy.deepcopy(VOLUME_STATS_RESP)
         expected["storage_protocol"] = 'DRBD'
+        expected["pools"][0]['location_info'] = (
+            'LinstorDrbdDriver:' + expected["pools"][0]['location_info'])
         self.assertEqual(expected, val)
 
     @mock.patch(DRIVER + 'linstor')
