@@ -43,7 +43,7 @@ NON_EXISTENT_IMAGE_ID = '003f540f-ec6b-4293-a3f9-7c68646b0f5c'
 
 
 class FakeImageService(object):
-    def __init__(self, db_driver=None, image_service=None):
+    def __init__(self, image_service=None):
         pass
 
     def show(self, context, image_id):
@@ -104,6 +104,28 @@ class CopyVolumeToImageTestCase(base.BaseVolumeTestCase):
         volume = db.volume_get(self.context, self.volume_id)
         self.assertEqual('available', volume['status'])
 
+    def test_copy_volume_to_image_with_conversion_disabled(self):
+        self.flags(image_conversion_disable=True)
+
+        self.volume_attrs['instance_uuid'] = None
+        volume_type_id = db.volume_type_create(
+            self.context, {'name': 'test', 'extra_specs': {
+                'image_service:store_id': 'fake_store'
+            }}).get('id')
+        self.volume_attrs['volume_type_id'] = volume_type_id
+        db.volume_create(self.context, self.volume_attrs)
+        image_meta = {
+            'id': self.image_id,
+            'container_format': 'ova',
+            'disk_format': 'vhdx'
+        }
+
+        self.assertRaises(exception.ImageConversionNotAllowed,
+                          self.volume.copy_volume_to_image,
+                          self.context,
+                          self.volume_id,
+                          image_meta)
+
     def test_copy_volume_to_image_over_image_quota(self):
         # creating volume testdata
         self.volume_attrs['instance_uuid'] = None
@@ -344,7 +366,7 @@ class CopyVolumeToImageTestCase(base.BaseVolumeTestCase):
     def test_copy_volume_to_image_with_image_volume(self):
         image = self._test_copy_volume_to_image_with_image_volume()
         self.assertTrue(image['locations'][0]['url'].startswith('cinder://'))
-        image_volume_id = image['locations'][0]['url'][9:]
+        image_volume_id = image['locations'][0]['url'].split('/')[-1]
         # The image volume does NOT include the snapshot_id, and include the
         # source_volid which is the uploaded-volume id.
         vol_ref = db.volume_get(self.context, image_volume_id)
@@ -481,14 +503,15 @@ class ImageVolumeTestCases(base.BaseVolumeTestCase):
     @mock.patch('cinder.quota.QUOTAS.reserve', return_value=["RESERVATION"])
     def test_clone_image_volume(self, mock_reserve, mock_commit,
                                 mock_rollback, mock_cloned_volume):
-        vol = tests_utils.create_volume(self.context,
+        # Confirm  cloning does not copy quota use field
+        vol = tests_utils.create_volume(self.context, use_quota=False,
                                         **self.volume_params)
         # unnecessary attributes should be removed from image volume
         vol.consistencygroup = None
         result = self.volume._clone_image_volume(self.context, vol,
                                                  {'id': fake.VOLUME_ID})
-
         self.assertNotEqual(False, result)
+        self.assertTrue(result.use_quota)  # Original was False
         mock_reserve.assert_called_once_with(self.context, volumes=1,
                                              volumes_vol_type_name=1,
                                              gigabytes=vol.size,
@@ -621,7 +644,7 @@ class ImageVolumeTestCases(base.BaseVolumeTestCase):
         mock_qemu_info.return_value = image_info
 
         def fake_copy_image_to_volume(context, volume, image_service,
-                                      image_id):
+                                      image_id, disable_sparse=False):
             raise exception.ImageTooBig(image_id=image_id, reason='')
 
         self.mock_object(self.volume.driver, 'copy_image_to_volume',
@@ -629,8 +652,8 @@ class ImageVolumeTestCases(base.BaseVolumeTestCase):
         self.assertRaises(exception.ImageTooBig,
                           self._create_volume_from_image)
 
-    @mock.patch('cinder.utils.brick_get_connector_properties')
-    @mock.patch('cinder.utils.brick_get_connector')
+    @mock.patch('cinder.volume.volume_utils.brick_get_connector_properties')
+    @mock.patch('cinder.volume.volume_utils.brick_get_connector')
     @mock.patch('cinder.volume.driver.BaseVD.secure_file_operations_enabled')
     @mock.patch('cinder.volume.driver.BaseVD._detach_volume')
     @mock.patch('cinder.image.image_utils.qemu_img_info')
@@ -656,8 +679,8 @@ class ImageVolumeTestCases(base.BaseVolumeTestCase):
         # We must have called detach method.
         self.assertEqual(1, mock_detach.call_count)
 
-    @mock.patch('cinder.utils.brick_get_connector_properties')
-    @mock.patch('cinder.utils.brick_get_connector')
+    @mock.patch('cinder.volume.volume_utils.brick_get_connector_properties')
+    @mock.patch('cinder.volume.volume_utils.brick_get_connector')
     @mock.patch('cinder.volume.driver.BaseVD._connect_device')
     @mock.patch('cinder.volume.driver.BaseVD._detach_volume')
     @mock.patch('cinder.image.image_utils.qemu_img_info')
