@@ -27,14 +27,12 @@ Volume driver library for NetApp 7/C-mode block storage systems.
 
 import copy
 import math
-import sys
 import uuid
 
 from oslo_log import log as logging
 from oslo_log import versionutils
 from oslo_utils import excutils
 from oslo_utils import units
-import six
 
 from cinder import exception
 from cinder.i18n import _
@@ -67,17 +65,19 @@ class NetAppLun(object):
 
     def __str__(self, *args, **kwargs):
         return 'NetApp LUN [handle:%s, name:%s, size:%s, metadata:%s]' % (
-               self.handle, self.name, self.size, self.metadata)
+            self.handle, self.name, self.size, self.metadata)
 
 
-@six.add_metaclass(volume_utils.TraceWrapperMetaclass)
-class NetAppBlockStorageLibrary(object):
+class NetAppBlockStorageLibrary(
+        object,
+        metaclass=volume_utils.TraceWrapperMetaclass):
     """NetApp block storage library for Data ONTAP."""
-
     # do not increment this as it may be used in volume type definitions
     VERSION = "1.0.0"
-    REQUIRED_FLAGS = ['netapp_login', 'netapp_password',
-                      'netapp_server_hostname']
+    REQUIRED_FLAGS_BASIC = ['netapp_login', 'netapp_password',
+                            'netapp_server_hostname']
+    REQUIRED_FLAGS_CERT = ['netapp_private_key_file',
+                           'netapp_certificate_file']
     ALLOWED_LUN_OS_TYPES = ['linux', 'aix', 'hpux', 'image', 'windows',
                             'windows_2008', 'windows_gpt', 'solaris',
                             'solaris_efi', 'netware', 'openvms', 'hyper_v']
@@ -96,6 +96,7 @@ class NetAppBlockStorageLibrary(object):
         self.driver_name = driver_name
         self.driver_protocol = driver_protocol
         self.zapi_client = None
+        self.dest_zapi_client = None
         self._stats = {}
         self.lun_table = {}
         self.lun_ostype = None
@@ -109,6 +110,8 @@ class NetAppBlockStorageLibrary(object):
         self.configuration = kwargs['configuration']
         self.configuration.append_config_values(na_opts.netapp_connection_opts)
         self.configuration.append_config_values(na_opts.netapp_basicauth_opts)
+        self.configuration.append_config_values(
+            na_opts.netapp_certificateauth_opts)
         self.configuration.append_config_values(na_opts.netapp_transport_opts)
         self.configuration.append_config_values(
             na_opts.netapp_provisioning_opts)
@@ -137,13 +140,18 @@ class NetAppBlockStorageLibrary(object):
         reserved_percentage = 100 * int(reserved_ratio)
         msg = ('The "netapp_size_multiplier" configuration option is '
                'deprecated and will be removed in the Mitaka release. '
-               'Please set "reserved_percentage = %d" instead.') % (
-                   reserved_percentage)
+               'Please set "reserved_percentage = %d" instead.') \
+            % reserved_percentage
         versionutils.report_deprecated_feature(LOG, msg)
         return reserved_percentage
 
     def do_setup(self, context):
-        na_utils.check_flags(self.REQUIRED_FLAGS, self.configuration)
+        if self.configuration.netapp_private_key_file or\
+                self.configuration.netapp_certificate_file:
+            na_utils.check_flags(self.REQUIRED_FLAGS_CERT,
+                                 self.configuration)
+        else:
+            na_utils.check_flags(self.REQUIRED_FLAGS_BASIC, self.configuration)
         self.lun_ostype = (self.configuration.netapp_lun_ostype
                            or self.DEFAULT_LUN_OS)
         self.host_type = (self.configuration.netapp_host_type
@@ -224,12 +232,17 @@ class NetAppBlockStorageLibrary(object):
 
         extra_specs = na_utils.get_volume_extra_specs(volume)
 
+        space_allocation = volume_utils.is_boolean_str(
+            extra_specs.get('netapp:space_allocation')
+        )
+        LOG.debug('create_volume space_allocation %r', space_allocation)
         lun_name = volume['name']
 
         size = int(volume['size']) * units.Gi
 
         metadata = {'OsType': self.lun_ostype,
                     'SpaceReserved': self.lun_space_reservation,
+                    'SpaceAllocated': str(space_allocation).lower(),
                     'Path': '/vol/%s/%s' % (pool_name, lun_name)}
 
         qos_policy_group_info = self._setup_qos_for_volume(volume, extra_specs)
@@ -237,9 +250,9 @@ class NetAppBlockStorageLibrary(object):
             na_utils.get_qos_policy_group_name_from_info(
                 qos_policy_group_info))
         qos_policy_group_is_adaptive = (volume_utils.is_boolean_str(
-            extra_specs.get('netapp:qos_policy_group_is_adaptive')) or
-            na_utils.is_qos_policy_group_spec_adaptive(
-                qos_policy_group_info))
+            extra_specs.get('netapp:qos_policy_group_is_adaptive'))
+            or na_utils.is_qos_policy_group_spec_adaptive
+            (qos_policy_group_info))
 
         try:
             self._create_lun(pool_name, lun_name, size, metadata,
@@ -362,9 +375,9 @@ class NetAppBlockStorageLibrary(object):
             na_utils.get_qos_policy_group_name_from_info(
                 qos_policy_group_info))
         qos_policy_group_is_adaptive = (volume_utils.is_boolean_str(
-            extra_specs.get('netapp:qos_policy_group_is_adaptive')) or
-            na_utils.is_qos_policy_group_spec_adaptive(
-                qos_policy_group_info))
+            extra_specs.get('netapp:qos_policy_group_is_adaptive'))
+            or na_utils.is_qos_policy_group_spec_adaptive
+            (qos_policy_group_info))
 
         try:
             self._clone_lun(
@@ -437,15 +450,17 @@ class NetAppBlockStorageLibrary(object):
                         " host OS.",
                         {'ig_nm': igroup_name, 'ig_os': ig_host_os})
         try:
-            return self.zapi_client.map_lun(path, igroup_name, lun_id=lun_id)
-        except netapp_api.NaApiError:
-            exc_info = sys.exc_info()
+            result = self.zapi_client.map_lun(path, igroup_name, lun_id=lun_id)
+            if self._is_active_sync_configured(self.configuration):
+                self.dest_zapi_client.map_lun(path, igroup_name, lun_id=lun_id)
+            return result
+        except netapp_api.NaApiError as e:
             (_igroup, lun_id) = self._find_mapped_lun_igroup(path,
                                                              initiator_list)
             if lun_id is not None:
                 return lun_id
             else:
-                six.reraise(*exc_info)
+                raise e
 
     def _unmap_lun(self, path, initiator_list):
         """Unmaps a LUN from given initiator."""
@@ -462,6 +477,8 @@ class NetAppBlockStorageLibrary(object):
 
         for _path, _igroup_name in lun_unmap_list:
             self.zapi_client.unmap_lun(_path, _igroup_name)
+            if self._is_active_sync_configured(self.configuration):
+                self.dest_zapi_client.unmap_lun(_path, _igroup_name)
 
     def _find_mapped_lun_igroup(self, path, initiator_list):
         """Find an igroup for a LUN mapped to the given initiator(s)."""
@@ -471,6 +488,21 @@ class NetAppBlockStorageLibrary(object):
         """Checks whether any LUNs are mapped to the given initiator(s)."""
         return self.zapi_client.has_luns_mapped_to_initiators(initiator_list)
 
+    def _is_active_sync_configured(self, config):
+        backend_names = []
+        replication_devices = config.safe_get('replication_device')
+        if replication_devices:
+            for replication_device in replication_devices:
+                backend_id = replication_device.get('backend_id')
+                if backend_id:
+                    backend_names.append(backend_id)
+
+        replication_enabled = True if backend_names else False
+        if replication_enabled:
+            return config.safe_get('netapp_replication_policy') == \
+                "AutomatedFailOver"
+        return False
+
     def _get_or_create_igroup(self, initiator_list, initiator_group_type,
                               host_os_type):
         """Checks for an igroup for a set of one or more initiators.
@@ -491,16 +523,36 @@ class NetAppBlockStorageLibrary(object):
         else:
             igroup_name = self._create_igroup_add_initiators(
                 initiator_group_type, host_os_type, initiator_list)
+        if self._is_active_sync_configured(self.configuration):
+            igroups_dest = self.dest_zapi_client.get_igroup_by_initiators(
+                initiator_list)
+            for igroup in igroups_dest:
+                igroup_name_dest = igroup['initiator-group-name']
+                if igroup_name_dest.startswith(na_utils.OPENSTACK_PREFIX):
+                    host_os_type = igroup['initiator-group-os-type']
+                    initiator_group_type = igroup['initiator-group-type']
+                    break
+            else:
+                self._create_igroup_add_initiators(
+                    initiator_group_type, host_os_type, initiator_list)
+
         return igroup_name, host_os_type, initiator_group_type
 
     def _create_igroup_add_initiators(self, initiator_group_type,
                                       host_os_type, initiator_list):
         """Creates igroup and adds initiators."""
-        igroup_name = na_utils.OPENSTACK_PREFIX + six.text_type(uuid.uuid4())
+        igroup_name = na_utils.OPENSTACK_PREFIX + str(uuid.uuid4())
         self.zapi_client.create_igroup(igroup_name, initiator_group_type,
                                        host_os_type)
+        if self._is_active_sync_configured(self.configuration):
+            self.dest_zapi_client.create_igroup(igroup_name,
+                                                initiator_group_type,
+                                                host_os_type)
         for initiator in initiator_list:
             self.zapi_client.add_igroup_initiator(igroup_name, initiator)
+            if self._is_active_sync_configured(self.configuration):
+                self.dest_zapi_client.add_igroup_initiator(igroup_name,
+                                                           initiator)
         return igroup_name
 
     def _delete_lun_from_table(self, name):
@@ -600,8 +652,8 @@ class NetAppBlockStorageLibrary(object):
         name = volume['name']
         lun = self._get_lun_from_table(name)
         path = lun.metadata['Path']
-        curr_size_bytes = six.text_type(lun.size)
-        new_size_bytes = six.text_type(int(new_size) * units.Gi)
+        curr_size_bytes = str(lun.size)
+        new_size_bytes = str(int(new_size) * units.Gi)
         # Reused by clone scenarios.
         # Hence comparing the stored size.
         if curr_size_bytes == new_size_bytes:
@@ -712,6 +764,16 @@ class NetAppBlockStorageLibrary(object):
                               " post clone resize LUN %s.", seg[-1])
                     LOG.error("Exception details: %s", e)
 
+    def _is_space_alloc_enabled(self, path):
+        """Gets space allocation details for the LUN."""
+        LOG.debug("Getting LUN space allocation enabled.")
+        lun_infos = self.zapi_client.get_lun_by_args(path=path)
+        if not lun_infos:
+            seg = path.split('/')
+            msg = _('Failure getting LUN info for %s' % seg[-1])
+            raise exception.VolumeBackendAPIException(data=msg)
+        return lun_infos[0]['SpaceAllocated'] == "true"
+
     def _get_lun_block_count(self, path):
         """Gets block counts for the LUN."""
         LOG.debug("Getting LUN block count.")
@@ -828,8 +890,8 @@ class NetAppBlockStorageLibrary(object):
         LOG.info("Unmanaged LUN with current path %(path)s and uuid "
                  "%(uuid)s.",
                  {'path': managed_lun.get_metadata_property('Path'),
-                  'uuid': managed_lun.get_metadata_property('UUID')
-                  or 'unknown'})
+                  'uuid': managed_lun.get_metadata_property('UUID') or
+                  'unknown'})
 
     def initialize_connection_iscsi(self, volume, connector):
         """Driver entry point to attach a volume to an instance.
@@ -844,6 +906,7 @@ class NetAppBlockStorageLibrary(object):
         """
 
         initiator_name = connector['initiator']
+        lun_path = volume['provider_location'].split(':')[1]
         name = volume['name']
         lun_id = self._map_lun(name, [initiator_name], 'iscsi', None)
 
@@ -874,7 +937,7 @@ class NetAppBlockStorageLibrary(object):
         properties = na_utils.get_iscsi_connection_properties(lun_id, volume,
                                                               iqn, addresses,
                                                               ports)
-
+        properties['discard'] = self._is_space_alloc_enabled(lun_path)
         if self.configuration.use_chap_auth:
             chap_username, chap_password = self._configure_chap(initiator_name)
             self._add_chap_properties(properties, chap_username, chap_password)
@@ -1076,7 +1139,7 @@ class NetAppBlockStorageLibrary(object):
 
         # get WWPNs from controller and strip colons
         all_target_wwpns = self._get_fc_target_wwpns()
-        all_target_wwpns = [six.text_type(wwpn).replace(':', '')
+        all_target_wwpns = [str(wwpn).replace(':', '')
                             for wwpn in all_target_wwpns]
 
         target_wwpns = []
