@@ -26,6 +26,7 @@ import math
 import os
 import re
 import time
+import urllib
 
 from oslo_concurrency import processutils
 from oslo_config import cfg
@@ -33,8 +34,6 @@ from oslo_log import log as logging
 from oslo_log import versionutils
 from oslo_utils import netutils
 from oslo_utils import units
-import six
-from six.moves import urllib
 
 from cinder import context
 from cinder import exception
@@ -50,16 +49,15 @@ from cinder.volume.drivers.netapp import utils as na_utils
 from cinder.volume.drivers import nfs
 from cinder.volume import volume_utils
 
-
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 HOUSEKEEPING_INTERVAL_SECONDS = 600  # ten minutes
 
 
-@six.add_metaclass(volume_utils.TraceWrapperWithABCMetaclass)
 class NetAppNfsDriver(driver.ManageableVD,
                       driver.CloneableImageVD,
-                      nfs.NfsDriver):
+                      nfs.NfsDriver,
+                      metaclass=volume_utils.TraceWrapperWithABCMetaclass):
     """Base class for NetApp NFS driver for Data ONTAP."""
 
     # do not increment this as it may be used in volume type definitions
@@ -68,8 +66,10 @@ class NetAppNfsDriver(driver.ManageableVD,
     # ThirdPartySystems wiki page
     CI_WIKI_NAME = "NetApp_CI"
 
-    REQUIRED_FLAGS = ['netapp_login', 'netapp_password',
-                      'netapp_server_hostname']
+    REQUIRED_FLAGS_BASIC = ['netapp_login', 'netapp_password',
+                            'netapp_server_hostname']
+    REQUIRED_FLAGS_CERT = ['netapp_private_key_file',
+                           'netapp_certificate_file']
     DEFAULT_FILTER_FUNCTION = 'capabilities.utilization < 70'
     DEFAULT_GOODNESS_FUNCTION = '100 - capabilities.utilization'
 
@@ -82,6 +82,8 @@ class NetAppNfsDriver(driver.ManageableVD,
         super(NetAppNfsDriver, self).__init__(*args, **kwargs)
         self.configuration.append_config_values(na_opts.netapp_connection_opts)
         self.configuration.append_config_values(na_opts.netapp_basicauth_opts)
+        self.configuration.append_config_values(
+            na_opts.netapp_certificateauth_opts)
         self.configuration.append_config_values(na_opts.netapp_transport_opts)
         self.configuration.append_config_values(na_opts.netapp_img_cache_opts)
         self.configuration.append_config_values(na_opts.netapp_nfs_extra_opts)
@@ -91,7 +93,12 @@ class NetAppNfsDriver(driver.ManageableVD,
     def do_setup(self, context):
         super(NetAppNfsDriver, self).do_setup(context)
         self._context = context
-        na_utils.check_flags(self.REQUIRED_FLAGS, self.configuration)
+        if self.configuration.netapp_private_key_file or\
+                self.configuration.netapp_certificate_file:
+            na_utils.check_flags(self.REQUIRED_FLAGS_CERT,
+                                 self.configuration)
+        else:
+            na_utils.check_flags(self.REQUIRED_FLAGS_BASIC, self.configuration)
         self.zapi_client = None
 
     def check_for_setup_error(self):
@@ -496,11 +503,13 @@ class NetAppNfsDriver(driver.ManageableVD,
         """Get the default goodness_function string."""
         return self.DEFAULT_GOODNESS_FUNCTION
 
-    def copy_image_to_volume(self, context, volume, image_service, image_id):
+    def copy_image_to_volume(self, context, volume, image_service, image_id,
+                             disable_sparse=False):
         """Fetch the image from image_service and write it to the volume."""
         self._ensure_flexgroup_not_in_cg(volume)
         super(NetAppNfsDriver, self).copy_image_to_volume(
-            context, volume, image_service, image_id)
+            context, volume, image_service, image_id,
+            disable_sparse=disable_sparse)
         LOG.info('Copied image to volume %s using regular download.',
                  volume['id'])
 
@@ -541,6 +550,7 @@ class NetAppNfsDriver(driver.ManageableVD,
 
     def _do_clone_rel_img_cache(self, src, dst, share, cache_file):
         """Do clone operation w.r.t image cache file."""
+
         @utils.synchronized(cache_file, external=True)
         def _do_clone():
             dir = self._get_mount_point_for_share(share)
@@ -551,6 +561,7 @@ class NetAppNfsDriver(driver.ManageableVD,
                                                     share=share)
                 src_path = '%s/%s' % (dir, src)
                 os.utime(src_path, None)
+
         _do_clone()
 
     def _clean_image_cache(self):
@@ -947,7 +958,7 @@ class NetAppNfsDriver(driver.ManageableVD,
             exception_msg = (_("Failed to extend volume "
                                "%(name)s, Error msg: %(msg)s.") %
                              {'name': volume['name'],
-                              'msg': six.text_type(err)})
+                              'msg': str(err)})
             raise exception.VolumeBackendAPIException(data=exception_msg)
 
         try:
@@ -962,7 +973,7 @@ class NetAppNfsDriver(driver.ManageableVD,
             exception_msg = (_("Failed to set QoS for existing volume "
                                "%(name)s, Error msg: %(msg)s.") %
                              {'name': volume['name'],
-                              'msg': six.text_type(err)})
+                              'msg': str(err)})
             raise exception.VolumeBackendAPIException(data=exception_msg)
 
     def _is_share_clone_compatible(self, volume, share):
@@ -1160,7 +1171,7 @@ class NetAppNfsDriver(driver.ManageableVD,
             exception_msg = (_("Failed to set QoS for existing volume "
                                "%(name)s, Error msg: %(msg)s.") %
                              {'name': existing_vol_ref['source-name'],
-                              'msg': six.text_type(err)})
+                              'msg': str(err)})
             raise exception.VolumeBackendAPIException(data=exception_msg)
 
         model_update = self._get_volume_model_update(volume) or {}
