@@ -15,12 +15,12 @@
 
 import json
 import posixpath
+import urllib
 
 from eventlet import greenthread
 from oslo_log import log as logging
 from oslo_utils.secretutils import md5
 import requests
-import six
 
 from cinder import exception
 from cinder.i18n import _
@@ -44,7 +44,7 @@ class NefException(exception.VolumeDriverException):
                     kwargs[key] = data[key]
                 else:
                     kwargs[key] = defaults[key]
-        elif isinstance(data, six.string_types):
+        elif isinstance(data, str):
             if 'message' not in kwargs:
                 kwargs['message'] = data
         for key in defaults:
@@ -328,7 +328,7 @@ class NefCollections(object):
         self.proxy = proxy
 
     def path(self, name):
-        quoted_name = six.moves.urllib.parse.quote_plus(name)
+        quoted_name = urllib.parse.quote_plus(name)
         return posixpath.join(self.root, quoted_name)
 
     def get(self, name, payload=None):
@@ -599,14 +599,14 @@ class NefProxy(object):
         prop = self.settings.get('system.guid')
         guid = prop.get('value')
         path = '%s:%s' % (guid, self.path)
-        if isinstance(path, six.text_type):
+        if isinstance(path, str):
             path = path.encode('utf-8')
         self.lock = md5(path, usedforsecurity=False).hexdigest()
 
     def url(self, path):
         netloc = '%s:%d' % (self.host, int(self.port))
         components = (self.scheme, netloc, str(path), None, None)
-        url = six.moves.urllib.parse.urlunsplit(components)
+        url = urllib.parse.urlunsplit(components)
         return url
 
     def delay(self, attempt):
