@@ -1,4 +1,4 @@
-#    (c) Copyright 2016 Brocade Communications Systems Inc.
+#    (c) Copyright 2019 Brocade, a Broadcom Company
 #    All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,14 +18,15 @@ Brocade south bound connector to communicate with switch using
 HTTP or HTTPS protocol.
 """
 
-from oslo_log import log as logging
-from oslo_serialization import base64
-import requests
-import six
 import time
 
-from cinder import exception
-from cinder.i18n import _, _LI
+from oslo_log import log as logging
+from oslo_serialization import base64
+from oslo_utils import encodeutils
+import requests
+
+from cinder.i18n import _
+from cinder.zonemanager.drivers.brocade import exception as b_exception
 import cinder.zonemanager.drivers.brocade.fc_zone_constants as zone_constant
 
 
@@ -61,6 +62,7 @@ class BrcdHTTPFCZoneClient(object):
         self.active_cfg = ''
         self.parsed_raw_zoneinfo = ""
         self.random_no = ''
+        self.auth_version = ''
         self.session = None
 
         # Create and assign the authentication header based on the credentials
@@ -81,7 +83,7 @@ class BrcdHTTPFCZoneClient(object):
         :param header: Request Headers
 
         :returns: HTTP response data
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
         try:
             if header is None:
@@ -97,7 +99,7 @@ class BrcdHTTPFCZoneClient(object):
                 adapter = requests.adapters.HTTPAdapter(pool_connections=1,
                                                         pool_maxsize=1)
                 self.session.mount(protocol + '://', adapter)
-            url = protocol + "://" + self.switch_ip + requestURL
+            url = '%s://%s%s' % (protocol, self.switch_ip, requestURL)
             response = None
             if requestType == zone_constant.GET_METHOD:
                 response = self.session.get(url,
@@ -114,7 +116,7 @@ class BrcdHTTPFCZoneClient(object):
                 msg = _("Error while querying page %(url)s on the switch, "
                         "reason %(error)s.") % {'url': url,
                                                 'error': response.reason}
-                raise exception.BrocadeZoningHttpException(msg)
+                raise b_exception.BrocadeZoningHttpException(msg)
             else:
                 return response.text
         except requests.exceptions.ConnectionError as e:
@@ -122,19 +124,19 @@ class BrcdHTTPFCZoneClient(object):
                      "with protocol %(protocol)s. Error: %(error)s.")
                    % {'switch_id': self.switch_ip,
                       'protocol': self.protocol,
-                      'error': six.text_type(e)})
+                      'error': str(e)})
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
-        except exception.BrocadeZoningHttpException as ex:
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
+        except b_exception.BrocadeZoningHttpException as ex:
             msg = (_("Unexpected status code from the switch %(switch_id)s "
                      "with protocol %(protocol)s for url %(page)s. "
                      "Error: %(error)s")
                    % {'switch_id': self.switch_ip,
                       'protocol': self.protocol,
                       'page': requestURL,
-                      'error': six.text_type(ex)})
+                      'error': str(ex)})
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def create_auth_token(self):
         """Create the authentication token.
@@ -143,7 +145,7 @@ class BrcdHTTPFCZoneClient(object):
         return authentication header (Base64(username:password:random no)).
 
         :returns: Authentication Header
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
         try:
             # Send GET request to secinfo.html to get random number
@@ -153,20 +155,27 @@ class BrcdHTTPFCZoneClient(object):
                                                zone_constant.SECINFO_BEGIN,
                                                zone_constant.SECINFO_END)
 
-            # Extract the random no from secinfo.html response
-            self.random_no = self.get_nvp_value(parsed_data,
-                                                zone_constant.RANDOM)
-            # Form the authentication string
-            auth_string = (self.switch_user + ":" + self.switch_pwd +
-                           ":" + self.random_no)
+            # Get the auth version for 8.1.0b+ switches
+            self.auth_version = self.get_nvp_value(parsed_data,
+                                                   zone_constant.AUTHVERSION)
+
+            if self.auth_version == "1":
+                # Extract the random no from secinfo.html response
+                self.random_no = self.get_nvp_value(parsed_data,
+                                                    zone_constant.RANDOM)
+                # Form the authentication string
+                auth_string = '%s:%s:%s' % (self.switch_user, self.switch_pwd,
+                                            self.random_no)
+            else:
+                auth_string = '%s:%s' % (self.switch_user, self.switch_pwd)
             auth_token = base64.encode_as_text(auth_string).strip()
             auth_header = (zone_constant.AUTH_STRING +
                            auth_token)  # Build the proper header
         except Exception as e:
             msg = (_("Error while creating authentication token: %s")
-                   % six.text_type(e))
+                   % str(e))
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
         return auth_header
 
     def authenticate(self):
@@ -176,7 +185,7 @@ class BrcdHTTPFCZoneClient(object):
         header (Base64(username:xxx:random no)).
 
         :returns: Authentication status
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
         headers = {zone_constant.AUTH_HEADER: self.auth_header}
         try:
@@ -190,9 +199,14 @@ class BrcdHTTPFCZoneClient(object):
             isauthenticated = self.get_nvp_value(
                 parsed_data, zone_constant.AUTHENTICATED)
             if isauthenticated == "yes":
-                # Replace password in the authentication string with xxx
-                auth_string = (self.switch_user +
-                               ":" + "xxx" + ":" + self.random_no)
+                if self.auth_version == "3":
+                    auth_id = self.get_nvp_value(parsed_data,
+                                                 zone_constant.IDENTIFIER)
+                    auth_string = '%s:xxx:%s' % (self.switch_user, auth_id)
+                else:
+                    # Replace password in the authentication string with xxx
+                    auth_string = '%s:xxx:%s' % (self.switch_user,
+                                                 self.random_no)
                 auth_token = base64.encode_as_text(auth_string).strip()
                 auth_header = zone_constant.AUTH_STRING + auth_token
                 return True, auth_header
@@ -201,12 +215,12 @@ class BrcdHTTPFCZoneClient(object):
                 msg = (_("Authentication failed, verify the switch "
                          "credentials, error code %s.") % auth_error_code)
                 LOG.error(msg)
-                raise exception.BrocadeZoningHttpException(reason=msg)
+                raise b_exception.BrocadeZoningHttpException(reason=msg)
         except Exception as e:
             msg = (_("Error while authenticating with switch: %s.")
-                   % six.text_type(e))
+                   % str(e))
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def get_session_info(self):
         """Get the session information from the switch
@@ -221,12 +235,12 @@ class BrcdHTTPFCZoneClient(object):
                                     header=headers)
         except Exception as e:
             msg = (_("Error while getting session information %s.")
-                   % six.text_type(e))
+                   % str(e))
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
         return response
 
-    def get_parsed_data(self, data, delim1, demil2):
+    def get_parsed_data(self, data, delim1, delim2):
         """Return the sub string between the delimiters.
 
         :param data: String to manipulate
@@ -237,12 +251,12 @@ class BrcdHTTPFCZoneClient(object):
         try:
             start = data.index(delim1)
             start = start + len(delim1)
-            end = data.index(demil2)
+            end = data.index(delim2)
             return data[start:end]
         except ValueError as e:
-            msg = (_("Error while parsing the data: %s.") % six.text_type(e))
+            msg = (_("Error while parsing the data: %s.") % str(e))
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def get_nvp_value(self, data, keyname):
         """Get the value for the key passed.
@@ -258,16 +272,16 @@ class BrcdHTTPFCZoneClient(object):
             end = temp.index("\n")
             return (temp[:end].lstrip('= '))
         except ValueError as e:
-            msg = (_("Error while getting nvp value: %s.") % six.text_type(e))
+            msg = (_("Error while getting nvp value: %s.") % str(e))
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def get_managable_vf_list(self, session_info):
         """List of VFIDs that can be managed.
 
         :param session_info: Session information from the switch
         :returns: manageable VF list
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
         try:
             # Check the value of manageableLFList NVP,
@@ -276,11 +290,11 @@ class BrcdHTTPFCZoneClient(object):
                                          zone_constant.MANAGEABLE_VF)
             if vf_list:
                 vf_list = vf_list.split(",")  # convert the string to list
-        except exception.BrocadeZoningHttpException as e:
+        except b_exception.BrocadeZoningHttpException as e:
             msg = (_("Error while checking whether "
-                     "VF is available for management %s.") % six.text_type(e))
+                     "VF is available for management %s.") % str(e))
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
         return vf_list[:-1]
 
     def change_vf_context(self, vfid, session_data):
@@ -288,7 +302,7 @@ class BrcdHTTPFCZoneClient(object):
 
         :param vfid: VFID to which context should be changed.
         :param session_data: Session information from the switch
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
         try:
             managable_vf_list = self.get_managable_vf_list(session_data)
@@ -309,11 +323,11 @@ class BrcdHTTPFCZoneClient(object):
                 session_LF_Id = self.get_nvp_value(parsed_info,
                                                    zone_constant.SESSION_LF_ID)
                 if session_LF_Id == vfid:
-                    LOG.info(_LI("VF context is changed in the session."))
+                    LOG.info("VF context is changed in the session.")
                 else:
                     msg = _("Cannot change VF context in the session.")
                     LOG.error(msg)
-                    raise exception.BrocadeZoningHttpException(reason=msg)
+                    raise b_exception.BrocadeZoningHttpException(reason=msg)
 
             else:
                 msg = (_("Cannot change VF context, "
@@ -321,11 +335,11 @@ class BrcdHTTPFCZoneClient(object):
                          "in the manageable VF list %(vf_list)s.")
                        % {'vf_list': managable_vf_list})
                 LOG.error(msg)
-                raise exception.BrocadeZoningHttpException(reason=msg)
-        except exception.BrocadeZoningHttpException as e:
-            msg = (_("Error while changing VF context %s.") % six.text_type(e))
+                raise b_exception.BrocadeZoningHttpException(reason=msg)
+        except b_exception.BrocadeZoningHttpException as e:
+            msg = (_("Error while changing VF context %s.") % str(e))
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def get_zone_info(self):
         """Parse all the zone information and store it in the dictionary."""
@@ -398,9 +412,9 @@ class BrcdHTTPFCZoneClient(object):
                 else:
                     i = i + 1
         except Exception as e:
-            msg = (_("Error while changing VF context %s.") % six.text_type(e))
+            msg = (_("Error while changing VF context %s.") % str(e))
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def is_supported_firmware(self):
         """Check firmware version is v6.4 or higher.
@@ -409,7 +423,7 @@ class BrcdHTTPFCZoneClient(object):
         This only checks major and minor version.
 
         :returns: True if firmware is supported else False.
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
 
         isfwsupported = False
@@ -436,9 +450,9 @@ class BrcdHTTPFCZoneClient(object):
 
         except Exception as e:
             msg = (_("Error while checking the firmware version %s.")
-                   % six.text_type(e))
+                   % str(e))
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
         return isfwsupported
 
     def get_active_zone_set(self):
@@ -459,7 +473,7 @@ class BrcdHTTPFCZoneClient(object):
                 'active_zone_config': 'OpenStack_Cfg'
             }
 
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
         active_zone_set = {}
         zones_map = {}
@@ -478,9 +492,9 @@ class BrcdHTTPFCZoneClient(object):
             return active_zone_set
         except Exception as e:
             msg = (_("Failed getting active zone set from fabric %s.")
-                   % six.text_type(e))
+                   % str(e))
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def add_zones(self, add_zones_info, activate, active_zone_set=None):
         """Add zone configuration.
@@ -505,7 +519,7 @@ class BrcdHTTPFCZoneClient(object):
         :param activate: True will activate the zone config.
         :param active_zone_set: Active zone set dict retrieved from
                                 get_active_zone_set method
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
         LOG.debug("Add zones - zones passed: %(zones)s.",
                   {'zones': add_zones_info})
@@ -540,7 +554,7 @@ class BrcdHTTPFCZoneClient(object):
                    % {'err_code': error_code, 'err_msg': error_msg})
 
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def update_zones(self, zone_info, activate, operation,
                      active_zone_set=None):
@@ -567,7 +581,7 @@ class BrcdHTTPFCZoneClient(object):
         :param operation: ZONE_ADD or ZONE_REMOVE
         :param active_zone_set: Active zone set dict retrieved from
                                 get_active_zone_set method
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
         LOG.debug("Update zones - zones passed: %(zones)s.",
                   {'zones': zone_info})
@@ -597,7 +611,7 @@ class BrcdHTTPFCZoneClient(object):
                    % {'err_code': error_code, 'err_msg': error_msg})
 
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def form_zone_string(self, cfgs, active_cfg,
                          zones, alias, qlps, ifas, activate):
@@ -611,7 +625,7 @@ class BrcdHTTPFCZoneClient(object):
         :param ifas: ifas map
         :param activate: True will activate config.
         :returns: zonestring in the required format
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
         try:
             zoneString = zone_constant.ZONE_STRING_PREFIX
@@ -620,19 +634,19 @@ class BrcdHTTPFCZoneClient(object):
             saveonly = "false" if activate is True else "true"
 
             # Form the zone string based on the dictionary of each items
-            for cfg in cfgs.keys():
+            for cfg in sorted(cfgs.keys()):
                 zoneString += (zone_constant.CFG_DELIM +
                                cfg + " " + cfgs.get(cfg) + " ")
-            for zone in zones.keys():
+            for zone in sorted(zones.keys()):
                 zoneString += (zone_constant.ZONE_DELIM +
                                zone + " " + zones.get(zone) + " ")
-            for al in alias.keys():
+            for al in sorted(alias.keys()):
                 zoneString += (zone_constant.ALIAS_DELIM +
                                al + " " + alias.get(al) + " ")
-            for qlp in qlps.keys():
+            for qlp in sorted(qlps.keys()):
                 zoneString += (zone_constant.QLP_DELIM +
                                qlp + " " + qlps.get(qlp) + " ")
-            for ifa in ifas.keys():
+            for ifa in sorted(ifas.keys()):
                 zoneString += (zone_constant.IFA_DELIM +
                                ifa + " " + ifas.get(ifa) + " ")
             # append the active_cfg string only if it is not null and activate
@@ -644,10 +658,11 @@ class BrcdHTTPFCZoneClient(object):
             zoneString += zone_constant.ZONE_END_DELIM + saveonly
         except Exception as e:
             msg = (_("Exception while forming the zone string: %s.")
-                   % six.text_type(e))
+                   % str(e))
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
-        return zoneString
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
+        # Reconstruct the zoneString to type base string for OpenSSL
+        return encodeutils.safe_encode(zoneString)
 
     def add_zones_cfgs(self, cfgs, zones, add_zones_info,
                        active_cfg, cfg_name):
@@ -690,9 +705,9 @@ class BrcdHTTPFCZoneClient(object):
         except Exception as e:
             msg = (_("Error while updating the new zones and cfgs "
                      "in the zone string. Error %(description)s.")
-                   % {'description': six.text_type(e)})
+                   % {'description': str(e)})
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
         return zones, cfgs, active_cfg
 
     def _update_zones(self, zones, updated_zones_info, operation):
@@ -726,9 +741,9 @@ class BrcdHTTPFCZoneClient(object):
         except Exception as e:
             msg = (_("Error while updating the zones "
                      "in the zone string. Error %(description)s.")
-                   % {'description': six.text_type(e)})
+                   % {'description': str(e)})
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
         return zones
 
     def is_vf_enabled(self):
@@ -743,7 +758,7 @@ class BrcdHTTPFCZoneClient(object):
         try:
             is_vf_enabled = bool(self.get_nvp_value(
                 parsed_data, zone_constant.VF_ENABLED))
-        except exception.BrocadeZoningHttpException:
+        except b_exception.BrocadeZoningHttpException:
             is_vf_enabled = False
             parsed_data = None
         return is_vf_enabled, parsed_data
@@ -761,15 +776,9 @@ class BrcdHTTPFCZoneClient(object):
         response = self.connect(zone_constant.GET_METHOD,
                                 zone_constant.NS_PAGE,
                                 header=headers)  # GET request to nsinfo.html
-        parsed_raw_zoneinfo = self.get_parsed_data(
-            response,
-            zone_constant.NSINFO_BEGIN,
-            zone_constant.NSINFO_END).strip("\t\n\r")
-        # build the name server information in the correct format
-        for line in parsed_raw_zoneinfo.splitlines():
-            start_index = line.find(zone_constant.NS_DELIM) + 7
-            if start_index != -1:
-                nsinfo.extend([line[start_index:start_index + 23].strip()])
+        for line in response.splitlines():
+            if line.startswith(zone_constant.NS_DELIM):
+                nsinfo.append(line.split('=')[-1])
         return nsinfo
 
     def delete_zones_cfgs(
@@ -786,7 +795,7 @@ class BrcdHTTPFCZoneClient(object):
         :param delete_zones_info: Zones map to add
         :param active_cfg: Existing active cfg
         :returns: updated zones, zone config sets, and active zone config
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
         try:
             delete_zones_info = delete_zones_info.split(";")
@@ -795,7 +804,7 @@ class BrcdHTTPFCZoneClient(object):
                 zones.pop(zone)
                 # iterated all the cfgs, but need to check since in SSH only
                 # active cfg is iterated
-                for k, v in cfgs.items():
+                for k, v in list(cfgs.items()):
                     v = v.split(";")
                     if zone in v:
                         # remove the zone from the cfg string
@@ -815,9 +824,9 @@ class BrcdHTTPFCZoneClient(object):
         except KeyError as e:
             msg = (_("Error while removing the zones and cfgs "
                      "in the zone string: %(description)s.")
-                   % {'description': six.text_type(e)})
+                   % {'description': str(e)})
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
         return zones, cfgs, active_cfg
 
     def delete_zones(self, delete_zones_info, activate, active_zone_set=None):
@@ -859,7 +868,7 @@ class BrcdHTTPFCZoneClient(object):
                      "(error code=%(err_code)s error msg=%(err_msg)s.")
                    % {'err_code': error_code, 'err_msg': error_msg})
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def post_zone_data(self, data):
         """Send POST request to the switch with the payload.
@@ -886,7 +895,7 @@ class BrcdHTTPFCZoneClient(object):
         timeout = 360
         sleep_time = 3
         time_elapsed = 0
-        while(status != "done"):
+        while (status != "done"):
             txn_response = self.connect(
                 zone_constant.GET_METHOD, transURL, "", headers)
             parsed_data_txn = self.get_parsed_data(txn_response,
@@ -920,19 +929,19 @@ class BrcdHTTPFCZoneClient(object):
             if self.vfid is None:
                 msg = _("No VF ID is defined in the configuration file.")
                 LOG.error(msg)
-                raise exception.BrocadeZoningHttpException(reason=msg)
+                raise b_exception.BrocadeZoningHttpException(reason=msg)
             elif self.vfid != 128:
                 self.change_vf_context(self.vfid, session_data)
         else:
             if self.vfid is not None:
                 msg = _("VF is not enabled.")
                 LOG.error(msg)
-                raise exception.BrocadeZoningHttpException(reason=msg)
+                raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def _disconnect(self):
         """Disconnect from the switch using HTTP/HTTPS protocol.
 
-        :raises: BrocadeZoningHttpException
+        :raises BrocadeZoningHttpException:
         """
         try:
             headers = {zone_constant.AUTH_HEADER: self.auth_header}
@@ -945,19 +954,19 @@ class BrcdHTTPFCZoneClient(object):
                      "with protocol %(protocol)s. Error: %(error)s.")
                    % {'switch_id': self.switch_ip,
                       'protocol': self.protocol,
-                      'error': six.text_type(e)})
+                      'error': str(e)})
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
-        except exception.BrocadeZoningHttpException as ex:
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
+        except b_exception.BrocadeZoningHttpException as ex:
             msg = (_("Unexpected status code from the switch %(switch_id)s "
                      "with protocol %(protocol)s for url %(page)s. "
                      "Error: %(error)s")
                    % {'switch_id': self.switch_ip,
                       'protocol': self.protocol,
                       'page': zone_constant.LOGOUT_PAGE,
-                      'error': six.text_type(ex)})
+                      'error': str(ex)})
             LOG.error(msg)
-            raise exception.BrocadeZoningHttpException(reason=msg)
+            raise b_exception.BrocadeZoningHttpException(reason=msg)
 
     def cleanup(self):
         """Close session."""
